from sympy import (Symbol, symbols, Matrix, sin, cos, asin, diff, sqrt, S,
                   diag, Eq, hessian, Function, flatten, Tuple, im, pi, latex,
                   dsolve, solve, fraction, factorial, Subs, Number, oo, Abs,
                   N, solveset)

from sympy.physics.mechanics import dynamicsymbols, ReferenceFrame, Point
from sympy.physics.vector import vpprint, vlatex
from ...dynamics import LagrangesDynamicSystem, HarmonicOscillator, mech_comp

from ..elements import MaterialPoint, Spring, GravitationalForce, Disk, RigidBody2D, Damper, PID, Excitation, Force, base_frame, base_origin
from  ..continuous import ContinuousSystem, PlaneStressProblem
from dynpy.models.mechanics.tmd import TunedMassDamper

import base64
import random
import IPython as IP
import numpy as np
import inspect

from .principles import ComposedSystem, NonlinearComposedSystem, base_frame, base_origin, REPORT_COMPONENTS_LIST

from functools import cached_property

from pint import UnitRegistry
ureg = UnitRegistry()

# class ComposedSystem(HarmonicOscillator):
#     """Base class for all systems

#     """
#     scheme_name = 'damped_car_new.PNG'
#     real_name = 'car_real.jpg'
#     detail_scheme_name = 'sruba_pasowana.png'
#     detail_real_name = 'buick_regal_3800.jpg'
#     _default_args = ()
#     _default_folder_path = "./dynpy/models/images/"
#     z = dynamicsymbols('z')
#     m0 = Symbol('m_0', positive=True)
#     k0 = Symbol('k_0', positive=True)
#     F0 = Symbol('F_0', positive=True)
#     Omega0 = Symbol('Omega_0', positive=True)

#     @classmethod
#     def _scheme(cls):

#         path = cls._default_folder_path + cls.scheme_name

#         return path

#     @classmethod
#     def _real_example(cls):
#         path = cls._default_folder_path + cls.real_name

#         return path

#     @classmethod
#     def _detail_real(cls):
#         path = cls._default_folder_path + cls.detail_real_name

#         return path

#     @classmethod
#     def _detail_scheme(cls):
#         path = cls._default_folder_path + cls.detail_scheme_name

#         return path

#     def _init_from_components(self,*args,system=None,**kwargs):
        
#         if system is None:
#             composed_system = self._elements_sum
#         else:
#             composed_system = system
            
#         #print('CS',composed_system._components)
#         super().__init__(None,system = composed_system)
        
#         #print('self',self._components)
#         if self._components is None:
#             comps = {}
#         else:
#             comps=self._components

#         self._components = {**comps,**self.components}

#     def __init__(self,
#                  Lagrangian=None,
#                  m0 = None,
#                  qs=None,
#                  forcelist=None,
#                  bodies=None,
#                  frame=None,
#                  hol_coneqs=None,
#                  nonhol_coneqs=None,
#                  label=None,
#                  ivar=None,
#                  evaluate=True,
#                  system=None):


#         if ivar is not None: self.ivar = ivar
#         if m0 is not None: self.m0 = m0

#         if qs is not None:
#             self.qs = qs
#         else:
#             self.qs = [self.z]


#         self._init_from_components(system=system)


#     @property
#     def components(self):

#         components = {}

#         self._material_point = MaterialPoint(self.m0, self.qs[0],
#                                              self.qs)('Material Point')
#         components['_material_point'] = self._material_point

#         return components

#     @property
#     def elements(self):
        
        
#         return {**super().components,**self.components}
    
#     @classmethod
#     def preview(cls, example=False):
#         if example:
#             path = cls._real_example()

#         elif example == 'detail_scheme_name':
#             path = cls._detail_scheme()
#         elif example == 'detail_real_name':
#             path = cls._detail_real()
#         else:
#             path = cls._scheme()
#         print(path)
#         with open(f"{path}", "rb") as image_file:
#             encoded_string = base64.b64encode(image_file.read())
#         image_file.close()

#         return IP.display.Image(base64.b64decode(encoded_string))

#     def get_default_data(self):
#         return None

#     def get_numerical_data(self):
#         return None
    
#     def get_random_parameters(self):

#         default_data_dict = self.get_default_data()

#         if default_data_dict:
#             parameters_dict = {
#                 key: random.choice(items_list)
#                 for key, items_list in default_data_dict.items()
#             }
#         else:
#             parameters_dict = None

#         return parameters_dict

#     def get_numerical_parameters(self):

#         default_data_dict = self.get_numerical_data()

#         if default_data_dict:
#             parameters_dict = {
#                 key: random.choice(items_list)
#                 for key, items_list in default_data_dict.items()
#             }
#         else:
#             parameters_dict = None

#         return parameters_dict
    
#     @property
#     def _report_components(self):

#         comp_list = [
#             mech_comp.TitlePageComponent,
#             mech_comp.SchemeComponent,
#             mech_comp.ExemplaryPictureComponent,
#             mech_comp.KineticEnergyComponent,
#             mech_comp.PotentialEnergyComponent,
#             mech_comp.LagrangianComponent,
#             mech_comp.GoverningEquationComponent,
#             mech_comp.FundamentalMatrixComponent,
#             mech_comp.GeneralSolutionComponent,
#             mech_comp.SteadySolutionComponent,
#         ]

#         return comp_list

    
#     def linearized(self):

#         return type(self).from_system(super().linearized())

# class CompoundSystem(ComposedSystem):

#     z = dynamicsymbols('z')
#     _p = Symbol('p')


#     @property
#     def components(self):

#         components = {}

#         self._material_point = MaterialPoint(self._p, self.qs[0],
#                                              self.qs)('Material Point')
#         components['_material_point'] = self._material_point

#         return components


# class NonlinearComposedSystem(ComposedSystem):

#     def frequency_response_function(self,
#                                     frequency=Symbol('Omega', positive=True),
#                                     amplitude=Symbol('a', positive=True)):

#         omega = ComposedSystem(self.linearized()).natural_frequencies()[0]
#         eps = self.small_parameter()

#         exciting_force = self.external_forces()[0]

#         comps = exciting_force.atoms(sin, cos)
#         exciting_amp = sum([exciting_force.coeff(comp) for comp in comps])
#         inertia = self.inertia_matrix()[0]

#         return amplitude * (-frequency**2 + omega**2) * inertia + S(
#             3) / 4 * eps * amplitude**3 - exciting_amp

#     def amplitude_from_frf(self, amplitude=Symbol('a', positive=True)):

#         return solveset(self.frequency_response_function(), amplitude)

#     @property
#     def _report_components(self):

#         comp_list = [
#             mech_comp.TitlePageComponent,
#             mech_comp.SchemeComponent,
#             mech_comp.ExemplaryPictureComponent,
#             mech_comp.KineticEnergyComponent,
#             mech_comp.PotentialEnergyComponent,
#             mech_comp.LagrangianComponent,
#             mech_comp.LinearizationComponent,
#             mech_comp.GoverningEquationComponent,
#             mech_comp.FundamentalMatrixComponent,
#             mech_comp.GeneralSolutionComponent,
#             mech_comp.SteadySolutionComponent,
#         ]

#         return comp_list


class BeamStructure(MaterialPoint):
    """
    Model of a Beam Structure composed of material point with changing point of mass:

    Creates a material point of an inertial body, after inputing correct values of mass -m and general coordinates, which follows a linear motion.
    """
    scheme_name = 'material_point.png'
    real_name = 'material_point.png'

    def get_default_data(self):
        m0=Symbol('m_0',positive=True)
        default_data_dict={
            self.m:[S.One * m0 * no for no in range(100, 200)],
        }
        return default_data_dict

    def get_numerical_data(self):

        self.default_data_dict={
            self.m:[no * 10000 for no in range(100, 200)],
        }
        return self.default_data_dict

class BeamElasticity(Spring):
    """
    Model of a Beam Elasticity: Creates a singular model, after inputing correct values of stiffeness - k_beam and general coordinate(s),
    which analytically display the dynamics of displacing spring after cummulating PE.
    """
    scheme_name =  'spring.PNG'
    real_name = 'spring_real.png'

    def get_default_data(self):
        E=Symbol('E',positive=True)
        I=Symbol('I',positive=True)
        l=Symbol('l',positive=True)
        default_data_dict={
            self.stiffness:[S.One * 48 * E * I / l**3 * no for no in range(50, 80)],

        }
        return default_data_dict

    def get_numerical_data(self):

        self.default_data_dict={
            self.stiffness:[no * 1000 for no in range(500, 800)],
        }
        return self.default_data_dict

# class BridgeExcitation(Force):
#     """
#     Creates enforcement.
#     """
#     scheme_name = ''
#     real_name = ''

#     def get_default_data(self):
#         F0=Symbol('F0',positive=True)
#         Omega0=Symbol('Omega0',positive=True)
#         default_data_dict={
#             self.force: [S.One * no * F0 * 1000 for no in range(10, 25)],
#         }
#         return default_data_dict

#     def get_numerical_data(self):

#         self.default_data_dict={
#             self.force:[no * 1000 for no in range(10, 25)],
#             self.Omega: [2 * 3.14 * 100],
#         }
#         return self.default_data_dict

#Amadi
#     """Ready to use model of bridge represented by the mass supported by elastic beam.
#         Arguments:
#         =========
#             m = Symbol object
#                 -Mass embedded on beam.

#             k = Symbol object
#                 -Bending stiffness of the beam

#             g = Symbol object
#                 -Gravitational field acceleration

#             ivar = symbol object
#                 -Independant time variable

#             qs = dynamicsymbol object
#                 -Generalized coordinates

#         Example
#         =======
#         A mass hanged on the elastic beam with the stiffness k in the gravitational field

#         >>> t = symbols('t')
#         >>> m, k = symbols('m, k')
#         >>> qs = dynamicsymbols('z') # Generalized Coordinates
#         >>> mass = SDoFHarmonicOscillator(m,k, qs=[z],) # Initialization of LagrangesDynamicSystem instance

#         -We define the symbols and dynamicsymbols
#         -Kinetic energy T and potential energy v are evaluated to calculate the lagrangian L
#         -Reference frame was created with point P defining the position and the velocity determined on the z axis
#         -external forces assigned
#         -Next we determine the instance of the system using class LagrangeDynamicSystem
#         -We call out the instance of the class
#         -If necessary assign values for the default arguments
#      #test

#     """
#     scheme_name = 'beam_bridge.PNG'
#     real_name = 'beam_bridge_real.PNG'

#     m = Symbol('m', positive=True)
#     k_beam = Symbol('k_beam', positive=True)
#     g = Symbol('g', positive=True)
#     Omega = Symbol('Omega', positive=True)
#     F = Symbol('F', positive=True)
#     z = dynamicsymbols('z')


#     def __init__(self,
#                  m=None,
#                  k_beam=None,
#                  ivar=Symbol('t'),
#                  g=None,
#                  Omega=None,
#                  F=None,
#                  z=None,
#                  **kwargs):

#         if m is not None: self.m = m
#         if k_beam is not None: self.k_beam = k_beam
#         if g is not None: self.g = g
#         if Omega is not None: self.Omega = Omega
#         if F is not None: self.F = F
#         if z is not None: self.z = z
#         self.ivar = ivar

#         self._init_from_components(**kwargs)

#     @cached_property
#     def components(self):

#         components = {}

#         self._mass = BeamStructure(self.m, self.z, qs=[self.z])(label='Mass of a beam')
#         self._spring = BeamElasticity(self.k_beam, self.z, qs=[self.z])(label='Beam stiffness')
#         self._gravity_force = GravitationalForce(self.m, self.g,self.z)(label='Gravity field')
#         self._force = Force(-self.F * sin(self.Omega * self.ivar),pos1=self.z)(label='External force')

#         components['_mass'] = self._mass
#         components['_spring'] = self._spring
#         components['_gravity_force'] = self._gravity_force
#         components['_force'] = self._force

#         return components

#     def symbols_description(self):
#         self.sym_desc_dict = {
#             self.m: r'Mass of system on the spring',
#             self.k_beam: r'Beam stiffness',
#             self.g: r'Gravitational field acceleration',
#             self.Omega: r'Excitation frequency',
#             self.F: r'Force acting on a bridge',
#             self.l: r'Lenght of a beam',
#             self.module: r'Youngs modulus',
#             self.inertia: r'Interia of a beam'
#         }

#         return self.sym_desc_dict

#     def get_default_data(self):
        
#         m0, k0, E, I, l, E0, I0= symbols('m_0 k_0 E I l E_0 I_0', positive=True)
        
#         F0=Symbol('F_0',positive=True)
#         Omega0=Symbol('Omega_0',positive=True)
#         E0=Symbol('E_0',positive=True)
#         I0=Symbol('I_0',positive=True)
#         l0=Symbol('l_0',positive=True)
#         module=Symbol('E',positive=True)
#         inertia=Symbol('I',positive=True)
#         l=Symbol('l',positive=True)
        
#         default_data_dict={
#             l: [S.One * no * l0 for no in range(1, 4)],
#             #module: [S.One * E0 * no for no in range(10, 20)],
#             #inertia: [S.One * no * I0 for no in range(10, 20)],
#             self.F: [S.One * no * F0  for no in range(10, 25)],
#             #self.m :[S.One * no * m0 for no in range(10, 20)]
# #            self.Omega: [S.One * Omega0]
#         }
#         return default_data_dict

#     def get_numerical_data(self):

#         default_data_dict={
#             self.F:[no * 1000 for no in range(10, 25)],
#             self.Omega: [3.14 * no for no in range(1,10)],
            
#         }
#         return default_data_dict

    
#     def dynamic_force(self):
#         data=self._given_data
#         amps=self._fodes_system.steady_solution.as_dict()
#         dyn_force=(self.components['_spring'].force().subs(amps)).subs(data).expand().doit()
        
#         return dyn_force
    
#     def static_force(self):
#         data=self._given_data
#         ans=self.dynamic_force()
#         free_coeff=ans.subs({cos(self.Omega*self.ivar):0, sin(self.Omega*self.ivar):0}).subs(data)
#         return (free_coeff)
    
class BeamBridge(ComposedSystem):
    """Ready to use model of bridge represented by the mass supported by elastic beam.
        Arguments:
        =========
            m = Symbol object
                -Mass embedded on beam.

            k = Symbol object
                -Bending stiffness of the beam

            g = Symbol object
                -Gravitational field acceleration

            ivar = symbol object
                -Independant time variable

            qs = dynamicsymbol object
                -Generalized coordinates

        Example
        =======
        A mass hanged on the elastic beam with the stiffness k in the gravitational field

        >>> t = symbols('t')
        >>> m, k = symbols('m, k')
        >>> qs = dynamicsymbols('z') # Generalized Coordinates
        >>> mass = SDoFHarmonicOscillator(m,k, qs=[z],) # Initialization of LagrangesDynamicSystem instance

        -We define the symbols and dynamicsymbols
        -Kinetic energy T and potential energy v are evaluated to calculate the lagrangian L
        -Reference frame was created with point P defining the position and the velocity determined on the z axis
        -external forces assigned
        -Next we determine the instance of the system using class LagrangeDynamicSystem
        -We call out the instance of the class
        -If necessary assign values for the default arguments
     #test

    """
    scheme_name = 'beam_bridge.PNG'
    real_name = 'beam_bridge_real.PNG'

    m = Symbol('m', positive=True)
    k_beam = Symbol('k_beam', positive=True)
    g = Symbol('g', positive=True)
    Omega = Symbol('Omega', positive=True)
    F = Symbol('F', positive=True)
    z = dynamicsymbols('z')


    def __init__(self,
                 m=None,
                 k_beam=None,
                 ivar=Symbol('t'),
                 g=None,
                 Omega=None,
                 F=None,
                 z=None,
                 **kwargs):

        if m is not None: self.m = m
        if k_beam is not None: self.k_beam = k_beam
        if g is not None: self.g = g
        if Omega is not None: self.Omega = Omega
        if F is not None: self.F = F
        if z is not None: self.z = z
        self.ivar = ivar

        self._init_from_components(**kwargs)

    @cached_property
    def components(self):

        components = {}

        self._mass = BeamStructure(self.m, self.z, qs=[self.z])(label='Mass of a beam')
        self._spring = BeamElasticity(self.k_beam, self.z, qs=[self.z])(label='Beam stiffness')
        self._gravity_force = GravitationalForce(self.m, self.g,self.z)(label='Gravity field')
        self._force = Force(-self.F * sin(self.Omega * self.ivar),pos1=self.z)(label='External force')

        components['_mass'] = self._mass
        components['_spring'] = self._spring
        components['_gravity_force'] = self._gravity_force
        components['_force'] = self._force

        return components

    def symbols_description(self):
        self.sym_desc_dict = {
            self.m: r'Mass of system on the spring',
            self.k_beam: r'Beam stiffness',
            self.g: r'Gravitational field acceleration',
            self.Omega: r'Excitation frequency',
            self.F: r'Force acting on a bridge',
            self.l: r'Lenght of a beam',
            self.module: r'Youngs modulus',
            self.inertia: r'Interia of a beam'
        }

        return self.sym_desc_dict

    def get_default_data(self):
        
        m0, k0, E, I, l, E0, I0= symbols('m_0 k_0 E I l E_0 I_0', positive=True)
        
        F0=Symbol('F_0',positive=True)
        Omega0=Symbol('Omega_0',positive=True)
        E0=Symbol('E_0',positive=True)
        I0=Symbol('I_0',positive=True)
        l0=Symbol('l_0',positive=True)
        module=Symbol('E',positive=True)
        inertia=Symbol('I',positive=True)
        l=Symbol('l',positive=True)
        
        default_data_dict={
            l: [S.One * no * l0 for no in range(1, 4)],
            #module: [S.One * E0 * no for no in range(10, 20)],
            #inertia: [S.One * no * I0 for no in range(10, 20)],
            self.F: [S.One * no * F0  for no in range(10, 25)],
            #self.m :[S.One * no * m0 for no in range(10, 20)]
#            self.Omega: [S.One * Omega0]
        }
        return default_data_dict

    def get_numerical_data(self):

        default_data_dict={
            self.F:[no * 1000 for no in range(10, 25)],
            self.Omega: [3.14 * no for no in range(1,10)],
            
        }
        return default_data_dict

    
    def dynamic_force(self):
        data=self._given_data
        amps=self._fodes_system.steady_solution.as_dict()
        dyn_force=(self.components['_spring'].force().subs(amps)).subs(data).expand().doit()
        
        return dyn_force
    
    def static_force(self):
        data=self._given_data
        ans=self.dynamic_force()
        free_coeff=ans.subs({cos(self.Omega*self.ivar):0, sin(self.Omega*self.ivar):0}).subs(data)
        return (free_coeff)
    
    def unit_dict(self):

        unit_dict = {

            self.k_beam: ureg.newton/ureg.meter,
            self.m: ureg.kilogram,
            self.g: ureg.meter/ureg.second/ureg.second,
            self.Omega: ureg.radian/ureg.second,
            self.F: ureg.newton,
            self.ivar: ureg.second,
            self.z: ureg.meter,
            self.z.diff(self.ivar): ureg.meter/ureg.second,
            self.z.diff(self.ivar,2): ureg.meter/ureg.second/ureg.second,
        }
        return unit_dict
    
    def reference_data_dict(self):

        E=Symbol('E',positive=True)
        I=Symbol('I',positive=True)
        l=Symbol('l',positive=True)

        default_data_dict={
            self.k_beam:[S.One * 48 * E * I / l**3 ],

        }
        return default_data_dict

    
#Amadi
class BeamBridgeDamped(BeamBridge):
    """Ready to use model of damped bridge represented by the mass supported by elastic beam.
        Arguments:
        =========
            m = Symbol object
                -Mass embedded on beam.

            k = Symbol object
                -Bending stiffness of the beam

            g = Symbol object
                -Gravitational field acceleration

            ivar = symbol object
                -Independant time variable

            qs = dynamicsymbol object
                -Generalized coordinates

        Example
        =======
        A mass hanged on the elastic beam with the stiffness k in the gravitational field

        >>> t = symbols('t')
        >>> m, k = symbols('m, k')
        >>> qs = dynamicsymbols('z') # Generalized Coordinates
        >>> mass = SDoFHarmonicOscillator(m,k, qs=[z],) # Initialization of LagrangesDynamicSystem instance

        -We define the symbols and dynamicsymbols
        -Kinetic energy T and potential energy v are evaluated to calculate the lagrangian L
        -Reference frame was created with point P defining the position and the velocity determined on the z axis
        -external forces assigned
        -Next we determine the instance of the system using class LagrangeDynamicSystem
        -We call out the instance of the class
        -If necessary assign values for the default arguments

    """
    scheme_name = 'bridge_dmp.png'
    real_name = 'beam_bridge_real.PNG'

    m = Symbol('m', positive=True)
    k_beam = Symbol('k_beam', positive=True)
    g = Symbol('g', positive=True)
    Omega = Symbol('Omega', positive=True)
    F = Symbol('F', positive=True)
    z = dynamicsymbols('z')
    c = Symbol('c', positive=True)


    def __init__(self,
                 m=None,
                 k_beam=None,
                 ivar=Symbol('t'),
                 g=None,
                 Omega=None,
                 F=None,
                 c=None,
                 z=None,
                 **kwargs):

        if m is not None: self.m = m
        if k_beam is not None: self.k_beam = k_beam
        if g is not None: self.g = g
        if Omega is not None: self.Omega = Omega
        if F is not None: self.F = F
        if z is not None: self.z = z
        if c is not None: self.c = c
        self.ivar = ivar

        self._init_from_components(**kwargs)

    @property
    def components(self):

        components = {}

        self._beam_bridge = BeamBridge(self.m, self.k_beam, self.ivar, self.g, self.Omega, self.F)(label='Beam Bridge')
        self._damper = Damper(self.c, pos1=self.z, qs=[self.z])(label='Damping of a beam')

        components['_beambridge'] = self._beam_bridge
        components['_damper'] = self._damper

        return components

    def symbols_description(self):
        self.sym_desc_dict = {
            self.m: r'Mass of system on the spring',
            self.k_beam: r'Beam stiffness',
            self.g: r'Gravitational field acceleration',
            self.Omega: r'Excitation frequency',
            self.F: r'Force acting on a bridge',
            self.l: r'Lenght of a beam',
            self.module: r'Youngs modulus',
            self.inertia: r'Interia of a beam',
            self.c: r'Damping coefficient'
        }

        return self.sym_desc_dict

    def get_default_data(self):

        lam=Symbol('lambda', positive = True)
        E, I, l=symbols('E I l',positive=True)
        F0=Symbol('F_0',positive=True)
        m0=Symbol('m_0',positive=True)
        k0=Symbol('k_0',positive=True)
        
        default_data_dict = {
#             self.k_beam [S.One * 48 * E * I / l**3],
            self.m: [S.One * m0 * no for no in range(100, 150)],
            self.c: [S.One * self.k_beam * lam],
            self.k_beam: [S.One * k0 * no for no in range(50, 100)],
            self.F: [S.One * F0 * no for no in range(10, 25)],
#             lam: [S.One * no * lam0 /1000 for no in range(1, 5)],
        }
        return default_data_dict
    

    def get_numerical_data(self):

        default_data_dict = {
            self.c:[S.One * no for no in range(5,20)],
        }
        return default_data_dict
    
    def dynamic_force(self):
        data=self._given_data
        amps=self._fodes_system.steady_solution.as_dict()
        dyn_force=(self._beam_bridge.components['_spring'].force().subs(amps)).subs(data).expand().doit()

        return dyn_force
    def unit_dict(self):

        from pint import UnitRegistry
        ureg=UnitRegistry()

        unit_dict = {
            self.m: ureg.kilogram,
            self.k_beam: ureg.newton/ureg.meter,
            self.g: ureg.meter/ureg.second/ureg.second,
            self.Omega: ureg.radian/ureg.second,
            self.F: ureg.newton,
            self.l: ureg.meter,
            self.module: ureg.newton/ureg.meter/ureg.meter,
            self.inertia: ureg.kilogram*ureg.meter*ureg.meter,
            self.c: ureg.newton*ureg.second/ureg.meter,
            self.z: ureg.meter,
            self.z.diff(self.ivar): ureg.meter/ureg.second,
            self.z.diff(self.ivar,2): ureg.meter/ureg.second/ureg.second,
        }
        return unit_dict
#Amadi
class BeamBridgeTMD(BeamBridge):

    scheme_name = 'bridge_tmd.png'
    real_name = 'beam_bridge_real.PNG'
    detail_real_name = 'real_bidge_tmd.jpg'

    m=Symbol('m', positive=True)
    m_TMD=Symbol('m_TMD', positive=True)
    k_beam=Symbol('k_beam', positive=True)
    k_TMD=Symbol('k_TMD', positive=True)
    g=Symbol('g', positive=True)
    Omega=Symbol('Omega', positive=True)
    F=Symbol('F', positive=True)
    z=dynamicsymbols('z')
    z_TMD=dynamicsymbols('z_TMD')

    
    def __init__(self,
                 m=None,
                 m_TMD=None,
                 k_beam=None,
                 k_TMD=None,
                 ivar=Symbol('t'),
                 g=None,
                 Omega=None,
                 F=None,
                 z=None,
                 z_TMD=None,
                 **kwargs):

        if m is not None: self.m = m
        if k_beam is not None: self.k_beam = k_beam
        if g is not None: self.g = g
        if Omega is not None: self.Omega = Omega
        if F is not None: self.F = F
        if m_TMD is not None: self.m_TMD = m_TMD
        if k_TMD is not None: self.k_TMD = k_TMD
        if z_TMD is not None: self.z_TMD = z_TMD
        if z is not None: self.z = z
        self.ivar=ivar
        
        self._init_from_components(**kwargs)

    @cached_property
    def components(self):

        components = {}

        self._beam_bridge = BeamBridge(self.m, self.k_beam, self.ivar, self.g, self.Omega, self.F)(label='Beam Bridge')
        self._TMD = TunedMassDamper(self.m_TMD, self.k_TMD, self.z_TMD,self.z)(label='TMD (damper)')
        self._TMD_gravity = GravitationalForce(self.m_TMD, self.g,self.z_TMD)(label='Gravity field for TMD')
        
        components['_beam_bridge'] = self._beam_bridge
        components['_TMD'] = self._TMD
        components['_TMD_gravity'] = self._TMD_gravity

        return components

    def symbols_description(self):
        self.sym_desc_dict = {
        self.m: r'Mass of beam',
        self.k_beam: r'Beam stiffness',
        self.g: r'Gravitational field acceleration',
        self.Omega: r'Excitation frequency',
        self.F: r'Force acting on a bridge',
        self.l: r'Lenght of a beam',
        self.module: r'Youngs modulus',
        self.inertia: r'Interia of a beam',
        self.m_TMD: r'Mass of TMD',
        self.k_TMD: r'Spring stiffness of a TMD'
        }

        return self.sym_desc_dict

    def get_default_data(self):

        m0, k0, E, I, l, E0, I0= symbols('m_0 k_0 E I l E_0 I_0', positive=True)

        default_data_dict = {
            self.m: [S.One * m0  * no for no in range(100, 200)],
            self.m_TMD: [S.One * m0/10  * no for no in range(100, 200)],
            self.k_TMD: [S.One * 48/10 * E * I / l**3 * no for no in range(50, 80)],
            self.k_beam: [S.One * 48 * E * I / l**3 * no for no in range(50, 80)],
#             E: [S.One * no * E0 for no in range(10,50)],
#             I: [S.One * no * I0 for no in range(20,80)],
        }

        return default_data_dict

    def get_random_parameters(self):

        default_data_dict = self.get_default_data()

        parameters_dict = {
            key: random.choice(items_list)
            for key, items_list in default_data_dict.items()
        }

        parameters_dict[self.m_TMD] = parameters_dict[self.m]/10
        parameters_dict[self.k_TMD] = parameters_dict[self.k_beam]/10

        return parameters_dict
    
    
    def get_numerical_data(self):

        default_data_dict = {
            self.m_TMD: [no * 1000 for no in range(100, 200)],
            self.k_TMD: [no * 100 for no in range(500, 800)],
            self.g: [9.81],

        }
        return default_data_dict

    def dynamic_force(self):
        
        '''This method counts dynamic force in in the beam of the investigated bridge without TMD. It works by substituting coordinates in beam force with steady solutions.'''
        
        force=BeamBridgeTMD().components['_beam_bridge'].components['_spring'].force()
        sol_dict=self._fodes_system.steady_solution.as_dict()
        F_kbeam=force.subs(sol_dict).subs(self._given_data)
        
        return F_kbeam.subs(self._given_data)
    
    def static_force(self):
        data=self._given_data
        ans=self.dynamic_force()
        free_coeff=ans.subs({cos(self.Omega*self.ivar):0, sin(self.Omega*self.ivar):0}).subs(data)
        return (free_coeff)
    
    def dynamic_force_TMD(self):
        '''This method counts dynamic force in spring between tuned mass damper and beam. It works by substituting coordinates in spring force with steady solutions.'''
        force=BeamBridgeTMD().components['_TMD'].components['_spring'].force()
        sol_dict=self._fodes_system.steady_solution.as_dict()
        F_tmd=force.subs(sol_dict).subs(self._given_data)
        
        return F_tmd
    
    def tmd_tune_coefficient(self):
        '''This method calculates the TMD tune coefficient. It assumes that nominator of steady solution is equal to 0 and then solves the equation with respect to k_TMD.'''
        
        pure_system = type(self)()
        
        Omega=pure_system.Omega
        t=self.ivar
        k_TMD=pure_system.k_TMD
        steady_z_tmd=pure_system._fodes_system.steady_solution.doit()
        coef_z_tmd=list(fraction(steady_z_tmd[0].coeff(sin(Omega*t))))
        sol_z_tmd=solve(Eq(coef_z_tmd[0], 0), k_TMD)
        
        return sol_z_tmd[0].subs(self._given_data)
    
    @property
    def _report_components(self):
        
        comp_list=[
        *REPORT_COMPONENTS_LIST,
#         mech_comp.FundamentalMatrixComponent,
#         mech_comp.MDoFGeneralSolutionComponent,
#         mech_comp.MDoFSteadySolutionComponent,
        mech_comp.TMDForceComponent

        ]

        return comp_list
    
class BeamBridgeDampedTMD(BeamBridge):

    scheme_name = 'bridge_tmd_dmp.png'
    real_name = 'beam_bridge_real.PNG'
    detail_real_name = 'real_bidge_tmd.jpg'

    c_TMD=Symbol('c_TMD', positive=True)
    c=Symbol('c', positive=True)
    m=Symbol('m', positive=True)
    m_TMD=Symbol('m_TMD', positive=True)
    k_beam=Symbol('k_beam', positive=True)
    k_TMD=Symbol('k_TMD', positive=True)
    g=Symbol('g', positive=True)
    Omega=Symbol('Omega', positive=True)
    F=Symbol('F', positive=True)
    z=dynamicsymbols('z')
    z_TMD=dynamicsymbols('z_TMD')

    def __init__(self,
                 m=None,
                 m_TMD=None,
                 k_beam=None,
                 k_TMD=None,
                 ivar=Symbol('t'),
                 g=None,
                 Omega=None,
                 F=None,
                 c_TMD=None,
                 c=None,
                 z_TMD=None,
                 z=None,
                 **kwargs):
        
        if m is not None: self.m = m
        if k_beam is not None: self.k_beam = k_beam
        if g is not None: self.g = g
        if Omega is not None: self.Omega = Omega
        if F is not None: self.F = F
        if m_TMD is not None: self.m_TMD = m_TMD
        if k_TMD is not None: self.k_TMD = k_TMD
        if z_TMD is not None: self.z_TMD = z_TMD
        if z is not None: self.z = z
        if c is not None: self.c = c
        if c_TMD is not None: self.c_TMD = c_TMD
        self.ivar=ivar
        
        self._init_from_components(**kwargs)

    @property
    def components(self):

        components = {}

        
        self._beam_bridge_damped = BeamBridgeDamped(self.m, self.k_beam, self.ivar, self.g, self.Omega, self.F, self.c)(label='Beam Bridge Damped')
        self._TMD = TunedMassDamper(self.m_TMD, self.k_TMD, self.z_TMD,self.z)(label='TMD (damper)')
        self._TMD_gravity = GravitationalForce(self.m_TMD, self.g,self.z_TMD)(label='Gravity field for TMD')
        self._damper_TMD = Damper(self.c_TMD, pos1=self.z - self.z_TMD, qs=[self.z, self.z_TMD])(label='Damper of a TMD')
        
        components['_beam_bridge_damped'] = self._beam_bridge_damped
        components['_TMD'] = self._TMD
        components['_TMD_gravity'] = self._TMD_gravity
        components['_damper_TMD'] = self._damper_TMD

        return components

    def symbols_description(self):
        self.sym_desc_dict = {
        self.m: r'Mass of beam',
        self.k_beam: r'Beam stiffness',
        self.g: r'Gravitational field acceleration',
        self.Omega: r'Excitation frequency',
        self.F: r'Force acting on a bridge',
        self.l: r'Lenght of a beam',
        self.module: r'Youngs modulus',
        self.inertia: r'Interia of a beam',
        self.m_TMD: r'Mass of TMD',
        self.k_TMD: r'Spring stiffness of a TMD',
        self.c: r'Damping coefficient of a bridge',
        self.c_TMD: r'Damping coefficient of a TMD'
        }

        return self.sym_desc_dict

    def get_default_data(self):
        lam=Symbol('lambda', positive = True)
        m0=Symbol('m_0', positive = True)
        k0=Symbol('k_0', positive = True)
        E, I = symbols('E I', positive=True)
        default_data_dict = {
            self.c_TMD: [S.One * self.k_TMD * lam],
            self.m_TMD: [S.One * no * m0 * 1000 for no in range(20, 80)],
            self.k_TMD: [S.One * no * (48 * E * I /self.l**3)* 1000 for no in range(10, 20)]
        }

        return default_data_dict

    def get_numerical_data(self):

        default_data_dict = {
            self.c_TMD:[S.One * no for no in range(5,10)],
            self.m_TMD: [S.One * no * 1000 for no in range(20, 80)],
            self.k_TMD: [S.One * no * 1000 for no in range(15, 20)]
        }
        return default_data_dict


from ..report import TikZPicture


class TuningTikZDiagram(TikZPicture):

    def _scheme_desc(self):

        code=r"""
\coordinate (origo) at (0,0);
\node (B1) [draw,outer sep=0pt,thick,minimum width=3.0cm, minimum height=1.0cm, align=center, rounded corners=0.5cm] at (origo) {Begin tuning};
\node (B2) [draw,outer sep=0pt,thick,minimum width=3.0cm, minimum height=1.0cm, align=center] at ([xshift=0cm,yshift=-1.5cm]B1.south) {Pump out the \\liquid from TMD to \\the skyscraper.};
\draw [thick, ->] ([yshift=0cm]B1.south) -- ([xshift=0cm,yshift=0cm]B2.north) node[midway,above] {};
\node (B3) [draw,outer sep=0pt,thick,minimum width=3.0cm, minimum height=1.0cm, align=center] at ([xshift=0cm,yshift=-2cm]B2.south) {Examine the \\natural frequency \\of the dynamic \\mass of the \\skyscraper.};
\draw [thick, ->] ([yshift=0cm]B2.south) -- ([xshift=0cm,yshift=0cm]B3.north) node[midway,above] {};
\node (B4) [draw,outer sep=0pt,thick,minimum width=3.0cm, minimum height=1.0cm, align=center] at ([xshift=0cm,yshift=-2.5cm]B3.south) {Calculate and \\transfer the \\amount of liquid \\that provides \\lower amplitudes \\of the damped \\structure.};
\draw [thick, ->] ([yshift=0cm]B3.south) -- ([xshift=0cm,yshift=0cm]B4.north) node[midway,above] {};
\node (B5) [draw,outer sep=0pt,thick,minimum width=3.0cm, minimum height=1.0cm, align=center, diamond, aspect=1.5] at ([xshift=0cm,yshift=-5.5cm]B4.south) {Compare the data with the model, \\is value of amplitude of linear acceleration \\of TMD equal or higher than in model?};
\draw [thick, ->] ([yshift=0cm]B4.south) -- ([xshift=0cm,yshift=0cm]B5.north) node[midway,above] {};
\node (B6) [draw,outer sep=0pt,thick,minimum width=3.0cm, minimum height=1.0cm, align=center] at ([xshift=4.2cm,yshift=0cm]B5.east) {TMD is tuned properly.};
\node (B7) [draw,outer sep=0pt,thick,minimum width=3.0cm, minimum height=1.0cm, align=center] at ([xshift=-4.2cm,yshift=0cm]B5.west) {Retune the TMD.};
\draw [thick, ->] ([xshift=0cm]B5.west) -- ([xshift=0cm,yshift=0cm]B7.east) node[midway,above] {Yes};
\draw [thick, ->] ([xshift=0cm]B5.east) -- ([xshift=0cm,yshift=0cm]B6.west) node[midway,above] {No};
\node (B8) [draw,outer sep=0pt,thick,minimum width=3.0cm, minimum height=1.0cm, align=center, rounded corners=0.5cm] at ([xshift=0cm,yshift=5cm]B7.north) {End tuning};
\draw [thick, ->] ([xshift=0cm]B7.north) -- ([xshift=0cm,yshift=0cm]B8.south) node[midway,above] {};
\draw [thick, ->] ([xshift=0cm]B6.north) |- ([xshift=0cm,yshift=0cm]B3.east) node[midway,above] {};
"""

        return code

class DiagnosticsTikZDiagram(TikZPicture):

    def _scheme_desc(self):

        code=r"""
\coordinate (origo) at (0,0);
\node (B1) [draw,outer sep=0pt,thick,minimum width=3.0cm, minimum height=1.0cm, align=center, rounded corners=0.5cm] at (origo) {Begin diagnostics};
\node (B2) [draw,outer sep=0pt,thick,minimum width=3.0cm, minimum height=1.0cm, align=center] at ([xshift=0cm,yshift=-1.5cm]B1.south) {Estimate the TMD's mass.};
\draw [thick, ->] ([yshift=0cm]B1.south) -- ([xshift=0cm,yshift=0cm]B2.north) node[midway,above] {};
\node (B3) [draw,outer sep=0pt,thick,minimum width=3.0cm, minimum height=1.0cm, align=center] at ([xshift=0cm,yshift=-3cm]B2.south) {Gain data from \\the linear \\acceleration sensor \\on the TMD};
\draw [thick, ->] ([yshift=0cm]B2.south) -- ([xshift=0cm,yshift=0cm]B3.north) node[midway,above] {};
\node (B4) [draw,outer sep=0pt,thick,minimum width=3.0cm, minimum height=1.0cm, align=center, diamond, aspect=1.5] at ([xshift=0cm,yshift=-3cm]B3.south) {$a_p {real} \geqslant  a_p {mod}$};
\draw [thick, ->] ([yshift=0cm]B3.south) -- ([xshift=0cm,yshift=0cm]B4.north) node[midway,above] {};
\node (B5) [draw,outer sep=0pt,thick,minimum width=3.0cm, minimum height=1.0cm, align=center] at ([xshift=4.2cm,yshift=0cm]B4.east) {Retune the TMD. \\Structure health \\is compromised.};
\node (B6) [draw,outer sep=0pt,thick,minimum width=3.0cm, minimum height=1.0cm, align=center] at ([xshift=-4.2cm,yshift=0cm]B4.west) {TMD is tuned properly. \\Structure health \\is not compromised.};
\draw [thick, ->] ([xshift=0cm]B4.west) -- ([xshift=0cm,yshift=0cm]B6.east) node[midway,above] {Yes};
\draw [thick, ->] ([xshift=0cm]B4.east) -- ([xshift=0cm,yshift=0cm]B5.west) node[midway,above] {No};
\node (B7) [draw,outer sep=0pt,thick,minimum width=3.0cm, minimum height=1.0cm, align=center] at ([xshift=0cm,yshift=5cm]B6.north) {Wait for another \\diagnostic interval.};
\draw [thick, ->] ([xshift=0cm]B6.north) -- ([xshift=0cm,yshift=0cm]B7.south) node[midway,above] {};
\node (B8) [draw,outer sep=0pt,thick,minimum width=3.0cm, minimum height=1.0cm, align=center, rounded corners=0.5cm] at ([xshift=0cm,yshift=5cm]B5.north) {End diagnostics};
\draw [thick, ->] ([xshift=0cm]B5.north) -- ([xshift=0cm,yshift=0cm]B8.south) node[midway,above] {};
\draw [thick, ->] ([xshift=0cm]B7.north) |- ([xshift=0cm,yshift=0cm]B1.west) node[midway,above] {};
"""

        return code
    
    
    
class TuningTikZDiagram1(TikZPicture):

    def _scheme_desc(self):

        
        code=r"""

    
    

    \draw
    (0,0) to[american voltage source, l_=$U_{oc}$] (0,-3)
    (0,0) to[R,european,-*,l=$R_{0}$] (6,0) node[label={right:$+$}] {}
    (0,-3) to[short, -*] (6,-3) node[label={right:$-$}] {}
;

"""

        return code


class TheveninTikZDiagramVer1(TikZPicture):

    def _scheme_desc(self):

        
        code=r"""

    
    \draw
    (0,0) to[american voltage source, l_=$U_{oc}$] (0,-2.5)
    (0,0) to[R,european, l=$R_{0}$] (3,0)
    
    (3,0) -- (3,1.5)
    to[C, l=$C_{1}$] (6,1.5) -- (6,0)
    (3,0) to[R,european, l=$R_{1}$] (6,0)
    
    (6,0) to[short, -*] (7,0) node[label={right:$+$}] {}
    
    (0,-2.5) to[short, -*] (7,-2.5) node[label={right:$-$}] {}
;

"""

        return code


class TheveninTikZDiagramVer2(TikZPicture):

    def _scheme_desc(self):

        
        code=r"""

    \draw
    (0,0) to[american voltage source, l_=$U_{oc}$] (0,-3)
    (0,0) to[R,european, l=$R_{0}$] (3,0)
    
    (3,0) -- (3,1.5)
    to[C, l=$C_{1}$] (6,1.5) -- (6,0)
    (3,0) to[R,european, l=$R_{1}$] (6,0)
    
    (6,0) -- (7,0)
    
    (7,0) -- (7,1.5)
    to[C, l=$C_{2}$] (10,1.5) -- (10,0)
    (7,0) to[R,european, l=$R_{2}$] (10,0)
    
    
    
    (10,0) to[short, -*] (11,0) node[label={right:$+$}] {}
    
    (0,-3) to[short, -*] (11,-3) node[label={right:$-$}] {}
;

"""

        return code


class TheveninTikZDiagram(TikZPicture):

    def _scheme_desc(self):

        
        code=r"""

    
    \draw
    (0,0) to[american voltage source, l_=$U_{oc}$] (0,-3)
    (0,0) to[R,european, l=$R_{0}$] (3,0)
    
    (3,0) to[short, i_=$I_{TH}$] (3,1.5)
    to[C, l=$C_{1}$] (6,1.5) -- (6,0)
    (3,0) to[R,european, l=$R_{1}$,a= + $ U_{TH} $ -] (6,0)
    
    (6,0) to[short, -*, i_=$I_{L}$] (7,0) node[label={right:$+$}] {}
    
    (0,-3) to[short, -*] (7,-3) node[label={right:$-$}] {}
;
"""

        return code
    
    
class ForcedDampedSpringMassSystemScheme(TikZPicture):

    def _scheme_desc(self):

        
        code=r"""

\coordinate (origo) at (0,0);

\tikzstyle{spring}=[thick,decorate,decoration={zigzag,pre length=0.3cm,post length=0.3cm,segment length=0.3cm}]

\tikzstyle{damper}=[thick,decoration={markings,  
mark connection node=dmp,
mark=at position 0.5 with 
{
\node (dmp) [thick,inner sep=0pt,transform shape,rotate=-90,minimum width=15pt,minimum height=3pt,draw=none] {};
\draw [thick] ($(dmp.north east)+(2pt,0)$) -- (dmp.south east) -- (dmp.south west) -- ($(dmp.north west)+(2pt,0)$);
\draw [thick] ($(dmp.north)+(0,-5pt)$) -- ($(dmp.north)+(0,5pt)$);
}
}, decorate]

\tikzstyle{ground}=[fill,pattern=north east lines,draw=none,minimum width=0.75cm,minimum height=0.3cm]



%draw axes
%\fill[black] (origo) circle (0.05);


\node (M) [draw,outer sep=0pt,thick,minimum width=3cm, minimum height=1.5cm,yshift=2cm] at (origo) {$m$};
%\node (m1_label) at ($(M)+(0:0.5cm)$) {$m$};

\draw [spring]    ([xshift=-1cm]M.south) --++(0,-3cm) node (k_tire_end){} node[midway,left] {$k$};

    \draw [damper] ([xshift=1cm]M.south) --++(0,-3cm) node (c_tire_end){}  node[midway,right=0.25cm] {$c$};

%\fill [black] (M.center) circle (0.05);

% \node (beam) [fill=gray,anchor=north,xshift=0cm,yshift=0cm,minimum width=2cm,minimum height=0.05pt] at (origo) {} node[right]{$m_{beam}=0$};

% \draw [ultra thick] (k_sup_spring_left.west) -- (k_sup_spring_right.east) node[midway] (beam_center) {};


%         \node (road_sur) [fill=black,anchor=north,xshift=0cm,yshift=-2cm,minimum width=2cm,minimum height=0.05cm] at (beam) {};

%beam_center.center

%    \draw[spring] ([xshift=-0.0cm]k_sup_spring_left.center) -- ++(0,-1.5cm) node (k_tire_end) {} node[midway,left] {$k_{tire}$};

%        \draw[damper] ([xshift=+0.5cm](k_sup_spring_right.center) -- ++(0,-2cm) node (c_tire_end) {} node[midway,right=0.25cm] {$c_{tire}$};

\draw[ultra thick] ([xshift=-0.0cm]k_tire_end.west) -- ([xshift=+0.0cm]c_tire_end.east) node (force_attachment_point) {};


\draw [thin] (force_attachment_point) -- +(1.5,0cm) coordinate (force_leader_edn);
\draw [-latex,ultra thick] (force_leader_edn) -- +(0,1.5cm) node[above right] (force_t)  {$F(t)=F \cos(\Omega t)$};

"""

        return code
    
class ForcedSpringMassSystemScheme(TikZPicture):

    def _scheme_desc(self):

        
        code=r"""

\coordinate (origo) at (0,0);

\tikzstyle{spring}=[thick,decorate,decoration={zigzag,pre length=0.3cm,post length=0.3cm,segment length=0.3cm}]

\tikzstyle{damper}=[thick,decoration={markings,  
mark connection node=dmp,
mark=at position 0.5 with 
{
\node (dmp) [thick,inner sep=0pt,transform shape,rotate=-90,minimum width=15pt,minimum height=3pt,draw=none] {};
\draw [thick] ($(dmp.north east)+(2pt,0)$) -- (dmp.south east) -- (dmp.south west) -- ($(dmp.north west)+(2pt,0)$);
\draw [thick] ($(dmp.north)+(0,-5pt)$) -- ($(dmp.north)+(0,5pt)$);
}
}, decorate]

\tikzstyle{ground}=[fill,pattern=north east lines,draw=none,minimum width=0.75cm,minimum height=0.3cm]



%draw axes
%\fill[black] (origo) circle (0.05);


\node (M) [draw,outer sep=0pt,thick,minimum width=3cm, minimum height=1.5cm,yshift=2cm] at (origo) {$m$};
%\node (m1_label) at ($(M)+(0:0.5cm)$) {$m$};

\draw [spring]    ([xshift=0cm]M.south) --++(0,-3cm) node (k_tire_end){} node[midway,left] {$k$};

%    \draw [damper] ([xshift=1cm]M.south) --++(0,-3cm) node (c_tire_end){}  node[midway,right=0.25cm] {$c$};

%\fill [black] (M.center) circle (0.05);

% \node (beam) [fill=gray,anchor=north,xshift=0cm,yshift=0cm,minimum width=2cm,minimum height=0.05pt] at (origo) {} node[right]{$m_{beam}=0$};

% \draw [ultra thick] (k_sup_spring_left.west) -- (k_sup_spring_right.east) node[midway] (beam_center) {};


%         \node (road_sur) [fill=black,anchor=north,xshift=0cm,yshift=-2cm,minimum width=2cm,minimum height=0.05cm] at (beam) {};

%beam_center.center

%    \draw[spring] ([xshift=-0.0cm]k_sup_spring_left.center) -- ++(0,-1.5cm) node (k_tire_end) {} node[midway,left] {$k_{tire}$};

%        \draw[damper] ([xshift=+0.5cm](k_sup_spring_right.center) -- ++(0,-2cm) node (c_tire_end) {} node[midway,right=0.25cm] {$c_{tire}$};

\draw[ultra thick] ([xshift=-0.5cm]k_tire_end.west) -- ([xshift=+0.5cm]k_tire_end.east) node (force_attachment_point) {};


\draw [thin] (force_attachment_point) -- +(1.5,0cm) coordinate (force_leader_edn);
\draw [-latex,ultra thick] (force_leader_edn) -- +(0,1.5cm) node[above right] (force_t)  {$F(t)=F \cos(\Omega t)$};

"""

        return code
    
class GearModelOscillatorScheme(TikZPicture):

    def _scheme_desc(self):

        
        code=r"""

\coordinate (origo) at (0,0);

\tikzstyle{spring}=[thick,decorate,decoration={zigzag,pre length=0.3cm,post length=0.3cm,segment length=0.3cm}]

\tikzstyle{damper}=[thick,decoration={markings,  
mark connection node=dmp,
mark=at position 0.5 with 
{
\node (dmp) [thick,inner sep=0pt,transform shape,rotate=-90,minimum width=15pt,minimum height=3pt,draw=none] {};
\draw [thick] ($(dmp.north east)+(2pt,0)$) -- (dmp.south east) -- (dmp.south west) -- ($(dmp.north west)+(2pt,0)$);
\draw [thick] ($(dmp.north)+(0,-5pt)$) -- ($(dmp.north)+(0,5pt)$);
}
}, decorate]

\tikzstyle{ground}=[fill,pattern=north east lines,draw=none,minimum width=0.75cm,minimum height=0.3cm]



%draw axes
%\fill[black] (origo) circle (0.05);


\node (M) [draw,outer sep=0pt,thick,minimum width=3cm, minimum height=1.5cm,yshift=2cm] at (origo) {$m_{red}$};
%\node (m1_label) at ($(M)+(0:0.5cm)$) {$m$};

\draw [spring]    ([xshift=-1cm]M.south) --++(0,-3cm) node (k_tire_end){} node[midway,left] {$k_{m}$};

    \draw [damper] ([xshift=1cm]M.south) --++(0,-3cm) node (c_tire_end){}  node[midway,right=0.25cm] {$c_{m}$};

%\fill [black] (M.center) circle (0.05);

% \node (beam) [fill=gray,anchor=north,xshift=0cm,yshift=0cm,minimum width=2cm,minimum height=0.05pt] at (origo) {} node[right]{$m_{beam}=0$};

% \draw [ultra thick] (k_sup_spring_left.west) -- (k_sup_spring_right.east) node[midway] (beam_center) {};


%         \node (road_sur) [fill=black,anchor=north,xshift=0cm,yshift=-2cm,minimum width=2cm,minimum height=0.05cm] at (beam) {};

%beam_center.center

%    \draw[spring] ([xshift=-0.0cm]k_sup_spring_left.center) -- ++(0,-1.5cm) node (k_tire_end) {} node[midway,left] {$k_{m}$};

%        \draw[damper] ([xshift=+0.5cm](k_sup_spring_right.center) -- ++(0,-2cm) node (c_tire_end) {} node[midway,right=0.25cm] {$c_{m}$};

\draw[ultra thick] ([xshift=-0.0cm]k_tire_end.west) -- ([xshift=+0.0cm]c_tire_end.east) node (force_attachment_point) {};


\draw [thin] (force_attachment_point) -- +(0,0) coordinate (force_leader_edn);
\draw [-latex,ultra thick] (force_leader_edn) -- +(0,-1.5cm) node[above right] (force_t)  {$F$};

\draw [thin] (0,-2.8) -- +(0,1) coordinate (force_leader_edn) {} node[midway,left] {$x$};
\draw[->]        (0,-2)   -- (0,-2.8);

"""

        return code

class HelicalGearModelScheme(TikZPicture):

    def _scheme_desc(self):

        
        code=r"""
        
    \tikzstyle{spring}=[thick,decorate,decoration={zigzag,pre length=0.2cm,post length=0.2cm,segment length=0.2cm}]

    \tikzstyle{damper}=[thick,decoration={markings,  
    mark connection node=dmp,
    mark=at position 0.5 with 
    {
    \node (dmp) [thick,inner sep=0pt,transform shape,rotate=-90,minimum width=15pt,minimum height=3pt,draw=none] {};
    \draw [thick] ($(dmp.north east)+(2pt,0)$) -- (dmp.south east) -- (dmp.south west) -- ($(dmp.north west)+(2pt,0)$);
    \draw [thick] ($(dmp.north)+(0,-5pt)$) -- ($(dmp.north)+(0,5pt)$);
    }
    }, decorate]

    % Draw the first circle with radius 1
    \draw[thick] (0,0) circle (1);
    \node at (0,0) {$O_{1}$};
    \node at (0, -1.3) {$J_{1}$};
    \node at (0, -1.6) {$r_{b1}$};

    % Draw the second circle with radius 2
    \draw [thick] (6,0) circle (2);
    \node at (6,0) {$O_{2}$};
    \node at (6, -2.3) {$J_{2}$};
    \node at (6, -2.6) {$r_{b2}$};

    % Draw the curved arrow for the first circle
    \draw[->, red, thick] (-1.3,0) arc[start angle=-10, end angle=45, radius=-1];
    \node[red] at (-1.2,-1.2) {$T_1$};

    % Draw the curved arrow for the second circle
    \draw[->, red, thick] (8.5,0) arc[start angle=0, end angle=45, radius=2];
    \node[red] at (8.5,1) {$T_2$};
    
    %Drawing the phi1 coordinates arrows
    \draw[<-, thick] (-1.3,0) arc[start angle=-10, end angle=-45, radius=-1];
    \node  at (-1.6,0.6) {$\phi_{1}(t)$};
    
    %Drawing the phi2 coordinate arrow
    \draw[->, thick] (8.5,0) arc[start angle=0, end angle=-45, radius=2];
    \node  at (8.9,-0.6) {$\phi_{2}(t)$};
    
    %Drawing the spring and damper connection
   \draw [thick] (0.28,-0.97) -- (2.3,0.17);
   \draw [thick] (4.8,1.6) -- (3.37,0.78);
   \draw [spring]  (2.02,0.571) -- (3.06,1.212);
   \draw [damper]  (2.705,-0.383) -- (3.745,0.293);
   \draw (2.02,0.571) -- (2.705,-0.383);
   \draw (3.06,1.212) -- (3.745,0.293);
   \node at (2.5,1.4) {$k_{m}(t)$};
   \node at (3.5,-0.6) {$c_{m}(t)$};


    """
        return code
    
    
class HelicalGearVerticalModelScheme(TikZPicture):

    def _scheme_desc(self):

        
        code=r"""
        
    \tikzstyle{spring}=[thick,decorate,decoration={zigzag,pre length=0.2cm,post length=0.2cm,segment length=0.2cm}]

    \tikzstyle{damper}=[thick,decoration={markings,  
    mark connection node=dmp,
    mark=at position 0.5 with 
    {
    \node (dmp) [thick,inner sep=0pt,transform shape,rotate=-90,minimum width=15pt,minimum height=3pt,draw=none] {};
    \draw [thick] ($(dmp.north east)+(2pt,0)$) -- (dmp.south east) -- (dmp.south west) -- ($(dmp.north west)+(2pt,0)$);
    \draw [thick] ($(dmp.north)+(0,-5pt)$) -- ($(dmp.north)+(0,5pt)$);
    }
    }, decorate]

    % Draw the first circle with radius 1
    \draw[thick] (0,0) circle (1);
    \node at (0,0) {$O_{1}$};
    \node at (0, -1.3) {$J_{1}$};
    \node at (0, -1.6) {$r_{b1}$};

    % Draw the second circle with radius 2
    \draw [thick] (0,6) circle (2);
    \node at (0,6) {$O_{2}$};
    \node at (0, -2.3) {$J_{2}$};
    \node at (0, -2.6) {$r_{b2}$};

    % Draw the curved arrow for the first circle
    \draw[->, red, thick] (-1.3,0) arc[start angle=-10, end angle=45, radius=-1];
    \node[red] at (-1.2,-1.2) {$T_1$};

    % Draw the curved arrow for the second circle
    \draw[->, red, thick] (0,8.5) arc[start angle=0, end angle=45, radius=2];
    \node[red] at (1,8.5) {$T_2$};
    
    %Drawing the phi1 coordinates arrows
    \draw[<-, thick] (-1.3,0) arc[start angle=-10, end angle=-45, radius=-1];
    \node  at (-1.6,0.6) {$\phi_{1}(t)$};
    
    %Drawing the phi2 coordinate arrow
    \draw[->, thick] (1,8.5) arc[start angle=0, end angle=-45, radius=2];
    \node  at (-0.6,8.5) {$\phi_{2}(t)$};
    
    %Drawing the spring and damper connection
   %\draw [thick] (0.28,-0.97) -- (2.3,0.17);
   %\draw [thick] (4.8,1.6) -- (3.37,0.78);
   %\draw [spring]  (2.02,0.571) -- (3.06,1.212);
   %\draw [damper]  (2.705,-0.383) -- (3.745,0.293);
   %\draw (2.02,0.571) -- (2.705,-0.383);
   %\draw (3.06,1.212) -- (3.745,0.293);
   %\node at (2.5,1.4) {$k_{m}(t)$};
   %\node at (3.5,-0.6) {$c_{m}(t)$};


    """
        return code
    
    
    
#Created and/or developed by Ania Fila (AnnFyla) & Michał Kacperek (MichalKacperek)
class CompoundPendulumScheme2(TikZPicture):

    def _scheme_desc(self):

        code=r"""

    % Osie
    \draw[->] (-0.2,0) -- (4,0) node[right] {$x$};
    \draw[->] (0,0.2) -- (0,-4) node[below] {$y$};
    \node at (0.3,-1.7) {$\varphi$};

    
    % Wahadło obrócone
    \begin{scope}[rotate=25]
        \filldraw[fill=black!10, draw=black, thick] (0,0) -- (1.5,-3) arc[start angle=30, end angle=-190, radius=0.8] -- cycle;
        
    % Oś środka ciężkości
         \draw[dashed] (0,0) -- (1,-5);
         \filldraw[black] (0.5,-2.5) circle (1.5pt);

    % Oznaczenia
    \node at (0.8,-2.5) {$C$};
    \node at (2,-4.5) {$m, I, l$};
    %Moment wymuszający

    \end{scope}
    
    % Strzałka
    \draw[->] (0,-1.5) arc[start angle=-80,end angle=-50,radius=1.5];

    
    % Punkt zawieszenia bryły
    \fill (0,0) circle (2pt);
    


"""

        return code

    
#Created and/or developed by Ania Fila (AnnFyla) & Michał Kacperek (MichalKacperek)
class ForcedCompoundPendulumScheme2(TikZPicture):

    def _scheme_desc(self):

        code=r"""

       % Osie
    \draw[->] (-0.2,0) -- (4,0) node[right] {$x$};
    \draw[->] (0,0.2) -- (0,-4) node[below] {$y$};
    \node at (0.3,-1.7) {$\varphi$};

    
    % Wahadło obrócone
    \begin{scope}[rotate=25]
        \filldraw[fill=black!10, draw=black, thick] (0,0) -- (1.5,-3) arc[start angle=30, end angle=-190, radius=0.8] -- cycle;
        
    % Oś środka ciężkości
         \draw[dashed] (0,0) -- (1,-5);
         \filldraw[black] (0.5,-2.5) circle (1.5pt);

    % Oznaczenia
    \node at (0.8,-2.5) {$C$};
    \node at (2,-4.5) {$m, I, l$};
    %Moment wymuszający
    \node at (2.5,-3.4) {$M_0 \cos(\Omega  t)$};
    \draw[->] (0.1,-3.5) arc[start angle=-130,end angle=-40,radius=1];
    \end{scope}
    
    % Strzałka
    \draw[->] (0,-1.5) arc[start angle=-80,end angle=-50,radius=1.5];

    
    % Punkt zawieszenia bryły
    \fill (0,0) circle (2pt);
    

"""

        return code

class CompoundPendulumScheme(TikZPicture):

    def _scheme_desc(self):

        code=r"""

 % Osie
    \draw[->] (-0.2,0) -- (4,0) node[right] {$x$};
    \draw[->] (0,0.2) -- (0,-4) node[below] {$y$};
    \node at (0.2,-2) {$\varphi$};

    
    % Wahadło obrócone
    \begin{scope}[rotate=30]
        \filldraw[fill=black!10, draw=black, thick] (0,0) -- (1.5,-3) arc[start angle=-20, end angle=-160, radius=1.2] -- cycle;
         
         \draw[dashed] (0,0) -- (0,-4);
         \filldraw[black] (0,-2) circle (1.5pt);
    \end{scope}
    
    % Oznaczenia
    \node at (1.3,-1.7) {$C$};
    \node at (3.5,-3) {$m, I, l$};
    
    % Strzałka momentu zaczepiona na linii brzegowej bryły
    \draw[->] (0,-1.5) arc[start angle=-80,end angle=-50,radius=1.5];
   %\node at (3.8,-2.3) {$M_0 \cos(\Omega  t)$};
    %\draw[->] (1,-2.5) arc[start angle=-90,end angle=-20,radius=1];
    
    % Punkt zawieszenia bryły
    \fill (0,0) circle (2pt);


"""

        return code

    
#Created and/or developed by Ania Fila (AnnFyla) & Michał Kacperek (MichalKacperek)
class ForcedCompoundPendulumScheme(TikZPicture):

    def _scheme_desc(self):

        code=r"""

    % Osie
    \draw[->] (-0.2,0) -- (4,0) node[right] {$x$};
    \draw[->] (0,0.2) -- (0,-4) node[below] {$y$};
    \node at (0.2,-2) {$\varphi$};

    
    % Wahadło obrócone
    \begin{scope}[rotate=30]
        \filldraw[fill=black!10, draw=black, thick] (0,0) -- (1.5,-3) arc[start angle=-20, end angle=-160, radius=1.2] -- cycle;
         
         \draw[dashed] (0,0) -- (0,-4);
         \filldraw[black] (0,-2) circle (1.5pt);
    \end{scope}
    
    % Oznaczenia
    \node at (1.3,-1.7) {$C$};
    \node at (3.5,-3) {$m, I, l$};
    
    % Strzałka momentu zaczepiona na linii brzegowej bryły
    \draw[->] (0,-1.5) arc[start angle=-80,end angle=-50,radius=1.5];
    \node at (3.8,-2.3) {$M_0 \cos(\Omega  t)$};
    \draw[->] (1,-2.5) arc[start angle=-90,end angle=-20,radius=1];
    
    % Punkt zawieszenia bryły
    \fill (0,0) circle (2pt);


"""

        return code

    
    
#Created and/or developed by Ania Fila (AnnFyla) & Michał Kacperek (MichalKacperek)
class RollingHalfDiskScheme(TikZPicture):

    def _scheme_desc(self):

        code=r"""

    % Definicje
    \def\r{2}       % Promień półwalca
    \def\phi{-200}    % Kąt obrotu w stopniach
    % Półwalec obrócony o kąt \phi
    \begin{scope}[rotate around={\phi:(0,0)}]  % Rotacja wokół (0,0)
        \fill[gray!20] (-\r,0) arc[start angle=180, end angle=0, radius=\r] -- cycle;
        \draw[thick] (-\r,0) arc[start angle=180, end angle=0, radius=\r];
        \draw[thick] (-\r,0) -- (\r,0);
    \end{scope}

    % Środkiem masy
    \def\cx{0}  % Współrzędne C przed rotacją
    \def\cy{4*\r/3.1416} % Przybliżenie dla półwalca

    \begin{scope}[rotate around={\phi:(0,0)}]  % Rotacja punktu C
        \filldraw[black] (0,0.9) circle (1.5pt);

        \draw[dashed] (0,0) -- (\cx,2);

    % Kąt phi
    %\draw[->] (0,1.5) arc[start angle=0, end angle=60, radius=1];
    \draw[->] (0,1.5) arc[start angle=80,end angle=100,radius=1.5];
    \node at (-0.2,1.3) {$\varphi$};
    \end{scope}
    \node at (-0.55,-0.8) {$c$};

    % Oznaczenie masy i promienia
    \node at (-\r+0.3,1) {$m, r$};

    % Osie układu współrzędnych
    \draw[->] (-2.5,-2) -- (3,-2) node[right] {$x$};
    \draw[->] (-0.69,-2.5) -- (-0.69,1) node[above] {$y$};

    \draw[dashed] (0,0) -- (0,-2);


"""

        return code
    
    
#Created and/or developed by Ania Fila (AnnFyla) & Michał Kacperek (MichalKacperek)
class ForcedRollingHalfDiskScheme(TikZPicture):

    def _scheme_desc(self):

        code=r"""

    % Definicje
    \def\r{2}       % Promień półwalca
    \def\phi{-200}    % Kąt obrotu w stopniach
    % Półwalec obrócony o kąt \phi
    \begin{scope}[rotate around={\phi:(0,0)}]  % Rotacja wokół (0,0)
        \fill[gray!20] (-\r,0) arc[start angle=180, end angle=0, radius=\r] -- cycle;
        \draw[thick] (-\r,0) arc[start angle=180, end angle=0, radius=\r];
        \draw[thick] (-\r,0) -- (\r,0);
        \draw[->] (1,-0.1) arc[start angle=300,end angle=250,radius=2];
        
        
    \end{scope}

    % Środkiem masy
    \def\cx{0}  % Współrzędne C przed rotacją
    \def\cy{4*\r/3.1416} % Przybliżenie dla półwalca

    \begin{scope}[rotate around={\phi:(0,0)}]  % Rotacja punktu C
        \filldraw[black] (0,0.9) circle (1.5pt);

        \draw[dashed] (0,0) -- (\cx,2);

    % Kąt phi
    \draw[->] (0,1.5) arc[start angle=80,end angle=100,radius=1.5];
    
    \node at (-0.2,1.3) {$\varphi$};
    \end{scope}
    \node at (-0.55,-0.8) {$c$};

    % Oznaczenie masy i promienia
    \node at (-\r+0.3,1) {$m, r$};
    
    
    % Osie układu współrzędnych
    \draw[->] (-2.5,-2) -- (3,-2) node[right] {$x$};
    \draw[->] (-0.69,-2.5) -- (-0.69,1) node[above] {$y$};
    
    \draw[dashed] (0,0) -- (0,-2);
    \node at (0.4,0.8) {$M_0 \cos(\Omega  t)$};

"""

        return code
    
#Created and/or developed by Ania Fila (AnnFyla) & Michał Kacperek (MichalKacperek)
class RollingBarScheme(TikZPicture):

    def _scheme_desc(self):

        code=r"""

% Semi-cylinder (half-circle)
    \fill[gray!20] (-2,0) arc[start angle=180,end angle=0,radius=2];
    \draw (-2,0) arc[start angle=180,end angle=0,radius=2];
    \draw (-2,0) -- (2,0);
    
    % Inclined rectangular board, rotated by phi = -20 degrees, extended length
    \begin{scope}[rotate around={-20:(0,0)}]
        \draw[fill=gray!30] (-3.5,2) rectangle (2.5,2.3);
        % Center of mass dot
        \filldraw (-0.4,2.15) circle (0.05);
    \end{scope}
    
    % Coordinate axes (moved to be on top)
    \draw[->] (0,-0.5) -- (0,3) node[above] {$y$};
    \draw[->] (-2.5,0) -- (2.5,0) node[right] {$x$};
    
    % Rotation angle phi
    \draw[thick,->] (0,0) -- (0.7,1.9);
    \node at (0.2,1) {$\varphi$};
    
    % Angle arc
    \draw[thick,->] (0,1.5) arc[start angle=90,end angle=70,radius=1.5];
    
    % Center of mass
    \node at (0.5,2.6) {$C$};
    
    % Radius
    \node at (-1.5,0.3) {$r$};
    
    % Board parameters
    \node at (2.8,2.0) {$m,l,h$};

"""

        return code
    
#Created and/or developed by Ania Fila (AnnFyla) & Michał Kacperek (MichalKacperek)
class ForcedRollingBarScheme(TikZPicture):

    def _scheme_desc(self):

        code=r"""
    % Semi-cylinder (half-circle)
    \fill[gray!20] (-2,0) arc[start angle=180,end angle=0,radius=2];
    \draw (-2,0) arc[start angle=180,end angle=0,radius=2];
    \draw (-2,0) -- (2,0);

    
    % Inclined rectangular board, rotated by phi
    \begin{scope}[rotate around={-20:(0,0)}]
        \draw[fill=gray!30] (-3.5,2) rectangle (2.5,2.3);
        % Center of mass dot
        \filldraw (-0.4,2.15) circle (0.05);
        %\draw[-] (-0.4,2.15) -- (1,2)
        %\draw[->] (1,2) arc[start angle=90,end angle=70,radius=4];
    \end{scope}
    
    % Coordinate axes (moved to be on top)
    \draw[->] (0,-0.5) -- (0,3) node[above] {$y$};
    \draw[->] (-2.5,0) -- (2.5,0) node[right] {$x$};
    
    % Rotation angle phi
    \draw[thick,->] (0,0) -- (0.7,1.9);
    \node at (0.2,1) {$\varphi$};
    
    % Angle arc
    \draw[thick,->] (0,1.5) arc[start angle=90,end angle=70,radius=1.5];
 
    % Center of mass
    \node at (0.5,2.6) {$C$};
    
    % Radius
    \node at (-1.5,0.3) {$r$};
    
    % Board parameters
    \node at (2.8,2.0) {$m,l,h$};
    \node at (1.5,3.5) {$M_0 \cos(\Omega  t)$};
    \draw[->] (0.5,3) arc[start angle=90,end angle=70,radius=4];

"""

        return code
    

class TrolleyWithElasticPendulumScheme(TikZPicture):
    def _scheme_desc(self):
        code=r"""
\tikzstyle{spring}=[thick,decorate,decoration={zigzag,pre length=0.3cm,post length=0.3cm,segment length=0.3cm}]
\tikzstyle{damper}=[thick,decoration={markings,  
    mark connection node=dmp,
    mark=at position 0.5 with 
    {
    \node (dmp) [thick,inner sep=0pt,transform shape,rotate=-90,minimum width=15pt,minimum height=3pt,draw=none] {};
    \draw [thick] ($(dmp.north east)+(2pt,0)$) -- (dmp.south east) -- (dmp.south west) -- ($(dmp.north west)+(2pt,0)$);
    \draw [thick] ($(dmp.north)+(0,-5pt)$) -- ($(dmp.north)+(0,5pt)$);
    }
    }, decorate]
\tikzstyle{ground}=[fill,pattern=north east lines,draw=none,minimum width=0.75cm,minimum height=0.3cm]
\node (M) [draw,outer sep=0pt,thick,minimum width=3cm, minimum height=1.5cm] {$m_1$};
    \node (l1_label) at ($(M.south)+(305:2cm)$)   {$l_{1},k_{e}$};
    \draw[thick,gray,dashed] (M.south) -- ++(-90:2.5);
    \draw[spring,black] (M.south) --  ++(-70:3.5) coordinate (bob21) node[near end, right]{$m_{1}$};
    \fill (bob21) circle (0.2);
\node (ground) [ground,anchor=north west,xshift=-2.4cm,yshift=-0.25cm,minimum width=6cm] at (M.south west) {};
\node (wall_1) [ground,anchor=east,xshift=-2cm,yshift=0cm,minimum height=2cm ,minimum width=0.4cm] at (M.west) {};
\draw (wall_1.south east) -- (wall_1.north east);
\draw (wall_1.south east) -- (ground.north east);
\draw [thick] (M.south west) ++ (0.4cm,-0.125cm) circle (0.125cm)  (M.south east) ++ (-0.4cm,-0.125cm) circle (0.125cm);
\draw [spring]  (M.180) ++ (0cm,0.5cm) -- ($ (wall_1.0 )+(0cm,0.5cm)  $);
\draw [damper]  (M.180) ++ (0cm,-0.5cm) -- ($ (wall_1.0 )+(0cm,-0.5cm)  $);
\draw [thin] (M.east) (0,1.5) -- (0,0.75);
\draw [-latex,ultra thick] (M.east) ++ (-1.5cm,1.5cm) -- +(1.5cm,0cm);
\node (k1) at (-2.5cm,1cm) {$k_1$};
\node (c1) at (-2.5cm,0cm) {$c_1$};
\node(ft) at (0.75cm,1.75cm) {$f(t)$};

"""
        return code


class DoublePendulumMeasureScheme(TikZPicture):
    def _scheme_desc(self):
        code=r"""

\node (B1) [draw,outer sep=0pt,thick,minimum width=3.0cm, minimum height=1.0cm, align=center,] at (0,0) {Wymuszenie};
\node (B2) [draw,outer sep=0pt,thick,minimum width=3.0cm, minimum height=1.0cm, align=center] at ([xshift=4cm]B1.east) {Main member};
\node (UAcc) [draw,outer sep=0pt,thick,minimum width=3.0cm, minimum height=1.0cm, align=center] at ([xshift=4cm]B2.east) {Upper accelerometer};

\draw [thick, ->] (B1.east) -- (B2.west);
\draw [thick,double, ->] (B2.east) -- (UAcc.west);

\node (B3) [draw,outer sep=0pt,thick,minimum width=3.0cm, minimum height=1.0cm, align=center] at ([yshift=-4cm]B2.south) {Vibration absorber};

\node (LAcc) [draw,outer sep=0pt,thick,minimum width=3.0cm, minimum height=1.0cm, align=center] at ([yshift=-4cm]UAcc.south) {Lower accelerometer};

\draw [thick, <->] (B2.south) -- (B3.north);
\draw [thick,double, ->] (B3.east) -- (LAcc.west);

\node (DAQ) [draw,outer sep=0pt,thick,minimum width=3.0cm, minimum height=1.0cm, align=center] at ([yshift=-1.75cm]UAcc.south) {DAQ};

\draw [thick, ->] (LAcc.north) -- (DAQ);
\draw [thick, ->] (UAcc.south) -- (DAQ);
"""

        return code

# Created by Anna Mackojc
class ColouredDoublePendulumMeasureScheme(TikZPicture):
    def _scheme_desc(self):
        code=r"""

\definecolor{excitationColor}{RGB}{255, 102, 102} % Red
\definecolor{mainColor}{RGB}{102, 178, 255} % Blue
\definecolor{accColor}{RGB}{255, 178, 102} % Orange
\definecolor{ptmdColor}{RGB}{153, 204, 102} % Green
\definecolor{daqColor}{RGB}{204, 153, 255} % Purple

% Nodes
\node (B1) [draw, fill=excitationColor!30, thick, minimum width=3.2cm, minimum height=1.2cm, align=center, rounded corners] at (0,0) {\textbf{Excitation}};

\node (B2) [draw, fill=mainColor!30, thick, minimum width=3.2cm, minimum height=1.2cm, align=center, rounded corners] at ([xshift=4cm]B1.east) {\textbf{Main Member}};

\node (UAcc) [draw, fill=accColor!30, thick, minimum width=3.2cm, minimum height=1.2cm, align=center, rounded corners] at ([xshift=4cm]B2.east) {\textbf{Upper Accelerometer}};

% Arrows
\draw [thick, ->, >=latex, black] (B1.east) -- (B2.west);
\draw [thick, double, ->, >=latex, black] (B2.east) -- (UAcc.west);

% Vibration Absorber
\node (B3) [draw, fill=ptmdColor!30, thick, minimum width=3.2cm, minimum height=1.2cm, align=center, rounded corners] at ([yshift=-4cm]B2.south) {\textbf{Vibration Absorber (PTMD)}};

\node (LAcc) [draw, fill=accColor!30, thick, minimum width=3.2cm, minimum height=1.2cm, align=center, rounded corners] at ([yshift=-4cm]UAcc.south) {\textbf{Lower Accelerometer}};

% Arrows
\draw [thick, <->, >=latex, black] (B2.south) -- (B3.north);
\draw [thick, double, ->, >=latex, black] (B3.east) -- (LAcc.west);

% DAQ System
\node (DAQ) [draw, fill=daqColor!30, thick, minimum width=3.2cm, minimum height=1.2cm, align=center, rounded corners] at ([yshift=-1.75cm]UAcc.south) {\textbf{DAQ System}};

% Connections to DAQ
\draw [thick, ->, >=latex, black] (LAcc.north) -- (DAQ);
\draw [thick, ->, >=latex, black] (UAcc.south) -- (DAQ);
"""

        return code


# Created by Anna Mackojc
class SegmentBeamScheme(TikZPicture):
    def _scheme_desc(self):
        code=r"""

     \coordinate (origo) at (0,0);
     \coordinate (pivot) at (1,5);

    % draw axes
    \fill[black] (origo) circle (0.05);
    \draw[thick,gray,->] (origo) -- ++(15,0) node[black,right] {$x$};
    \draw[thick,gray,->] (origo) -- ++(0,4) node (mary) [black,right] {$y$};

    \filldraw[fill=blue!30, draw=black, thick, rotate around = {-30:(origo)}] (origo) rectangle ++ (3,0.25) coordinate (bob1) node[near end, below right]{};
    \filldraw[fill=blue!30, draw=black, thick, rotate around = {-90:(origo)}] (bob1) ++ (0,0) rectangle ++ (0.25,5) coordinate (bob2) node[anchor=north east]{};
    \filldraw[fill=blue!30, draw=black, thick, rotate around = {30:(origo)}] (bob2) ++ (0.15,-.05) rectangle ++ (3,0.25) coordinate (bob3) node[near end, right]{};

    \draw [shift={(bob1)},yshift=-5,domain=2*pi+310/180*pi:295/180*pi+7*pi,variable=\t,smooth,samples=75] plot ({\t r}: {0.0006*\t*\t});
    \draw [shift={(bob2)},xshift=1,yshift=4,domain=2*pi+315/180*pi:305/180*pi+7*pi,variable=\t,smooth,samples=75] plot ({\t r}: {0.0006*\t*\t});
"""
        return code
    


from sympy import (Symbol, symbols, Matrix, sin, cos, diff, sqrt, S, diag, Eq, Derivative,Integral, Expr,Function,latex, Heaviside, atan, pi)
from numbers import Number
from sympy.physics.mechanics import dynamicsymbols, ReferenceFrame, Point
from sympy.physics.vector import vpprint, vlatex

import numpy as np

from ...dynamics import LagrangesDynamicSystem, HarmonicOscillator, GeometryScene,base_frame,base_origin
from ...utilities.templates import tikz

#from dgeometry import GeometryScene,np

from pylatex import TikZ,TikZNode
import matplotlib.pyplot as plt

import base64
import random
import IPython as IP

from ...models.elements import Force,IntegralElement,ProportionalElement,DerivativeElement

class PIController(Force):
    
    '''
    A proportional–integral controller (PI controller) is a control loop mechanism
    employing feedback used in industrial control systems requiring continuously
    modulated control. A PI controller calculates an error value (e(t)) as the
    difference between a desired setpoint (SP) and a measured process variable (PV)
    and applies a correction based on proportional and integral terms. It consists of
    two independent elements - P and I that being combined return two-term controller.
    '''
    
    k_P = Symbol('k_P', positive=True)
    k_I = Symbol('k_I', positive=True)
    error = Symbol('e', positive=True)
    target = Symbol('target', positive=True)
    reference = S.Zero

    def __init__(self,
                 k_P,
                 k_I,
                 error = None,
                 target=None,
                 reference = None,
                 qs=None,
                 ivar=Symbol('t'),
                 frame = base_frame,
                 **kwargs):

        if k_P is not None: self.k_P = k_P
        if k_I is not None: self.k_I = k_I
        
        if reference is not None: self.reference = reference

        if error is not None: self.error = error

        if target is None and isinstance(self.error,Function):
               target=self.error

        self.ivar = ivar
        self.qs = qs

        self._init_from_components(**kwargs)

    @property
    def components(self):
        
        components = {}
        k_P, k_I = self.k_P, self.k_I
        
        self._proporional_elem = ProportionalElement(k_P, self.error, self.reference, qs = self.qs, ivar = self.ivar)
        self._integral_elem = IntegralElement(k_I, self.error, self.reference, qs = self.qs, ivar = self.ivar)
        
        components['proportional_elem'] = self._proporional_elem
        components['integral_elem'] = self._integral_elem
        
        return components


from sympy import (Symbol, symbols, Matrix, sin, cos, asin, diff, sqrt, S,
                   diag, Eq, hessian, Function, flatten, Tuple, im, pi, latex,
                   dsolve, solve, fraction, factorial, Subs, Number, oo, Abs,
                   N, solveset, integrate, lambdify, Heaviside, integrate, exp, atan, sign, sign)

from sympy.physics.mechanics import dynamicsymbols, ReferenceFrame, Point
from sympy.physics.vector import vpprint, vlatex
from ...dynamics import LagrangesDynamicSystem, HarmonicOscillator, mech_comp

from ..elements import MaterialPoint, Spring, GravitationalForce, Disk, RigidBody2D, Damper, PID, Excitation, Force, base_frame, base_origin, CombustionEngine
from ...solvers.linear import ODESystem

import pandas as pd

from .elements import *

import base64
import random
import IPython as IP
import numpy as np
import inspect


from ..mechanics.trolley import ComposedSystem, NonlinearComposedSystem, base_frame, base_origin

t = Symbol('t')

#from dynpy.models.electric import batterycell 
#mechanics_printing(pretty_print=True)




class BatteryCell(ComposedSystem):
    
    
    scheme_name = 'thevenincircuit.png'
    real_name = 'liioncell.PNG'

    R_1=Symbol('R_1', positive=True)
    R_2=Symbol('R_2', positive=True)

    L_1=Symbol('L_1', positive=True)
    L_2=Symbol('L_2', positive=True)
    
    C=Symbol('C', positive=True)
    U=Symbol('U', positive=True)
    q_1=dynamicsymbols('q_1')
    q_2=dynamicsymbols('q_2')
    t = Symbol('t')
    U_li = Function('U_li')(t)
    U_oc = Symbol('U_oc')
    R_0 = Symbol('R_0')
    I_li = Function('I_zad')(t)
    R_th = Symbol('R_th')
    C_th = Symbol('C_th')
    SOC = Function('SOC')(t)
    SOC_init = Symbol('SOC_init')
    C_rated = Symbol('C_rated')
    t_0 = Symbol('t_0')
    U_th = Function('U_th')(t)
    funcI= Function('funcI')(t)
    #U_th = Symbol('U_th')

    def __init__(self,
                 R_1=None,
                 R_2=None,
                 C=None,
                 U=None,
                 q_1=None,
                 q_2=None,
                 t=None,
                 U_th=None,
                 R_th=None,
                 C_th=None,
                 I_li=None,
                 ivar=Symbol('t'),
                 **kwargs):

        if t is not None: self.t = t
        if R_1 is not None: self.R_1 = R_1
        if R_2 is not None: self.R_2 = R_2
        if C is not None: self.C= C
        if U is not None: self.U = U
        if q_1 is not None: self.q_1 = q_1
        if q_2 is not None: self.q_2 = q_2
        if U_th is not None: self.U_th = U_th
        if R_th is not None: self.R_th = R_th
        if C_th is not None: self.C_th = C_th
        if I_li is not None: self.I_li = I_li
        self.qs = [self.q_1, self.q_2]
        self._init_from_components(**kwargs)


    @property
    def components(self):
        components = {}
        
        self._coil_1 = Inductor(self.L_1, self.q_1, qs=[self.q_1])
        self._coil_2 = Inductor(self.L_2, self.q_2, qs=[self.q_2])
        
        self._resistor_1 = Resistor(self.R_1, self.q_1, qs=[self.q_1])
        self._resistor_2 = Resistor(self.R_2, q0=self.q_2, qs=[self.q_2])
        self._voltagesource = VoltageSource(self.U, q0 = self.q_1, qs=[self.q_1])
        self._capacitor = Capacitor(self.C, q0 = self.q_1-self.q_2, qs=[self.q_1, self.q_2])

        
        components['coil_1'] = self._coil_1
        components['coil_2'] = self._coil_2
        
        components['resistor_1'] = self._resistor_1
        components['resistor_2'] = self._resistor_2
        components['voltagesource'] = self._voltagesource
        components['capacitor'] = self._capacitor

        return components


    def voltage_ode(self):

        t = self.ivar
        U_th = self.U_th
        R_th = self.R_th
        C_th = self.C_th
        I_li = self.I_li
        
        
        
        
        uth_eq=Eq(diff(U_th,t, evaluate=False),((U_th)/(R_th*C_th))+I_li/C_th)
        ode_th = ODESystem(odes=uth_eq.rhs+uth_eq.lhs, dvars=U_th, ode_order=1)
        
        return ode_th
    
    
    def uth_simulation(self,data_dict1,celldata, step_time,step_current):
        
        t = self.ivar
        U_th = self.U_th
        R_th = self.R_th
        C_th = self.C_th
        I_li = self.I_li
        U_li = self.U_li
        U_oc = self.U_oc
        R_0 = self.R_0
        t_0 = self.t_0
        SOC = self.SOC
        C_rated = self.C_rated
        SOC_init = self.SOC_init
        ode_th = self.voltage_ode()
        
        

     
        #wzor na napiecie
        napiecie_eq=Eq(U_li,U_oc-R_0*I_li-U_th)

        
        #wzor na soc
        calka=integrate(I_li, (t, t_0, t))
        SOC_eq_bezcalki = Eq(SOC,((1)/(C_rated)))
        SOC_eq = Eq(SOC,SOC_init+((1)/(C_rated))*calka)
        
        
        #dane symulacji
        n_points=step_time[-1]*4 #rozdzielczosc symulacji probki na sekunde
        t_array=np.linspace(0,step_time[-1]*1,n_points)#czas symulacji i prad symulacji

        #tworzenie pustych list
        prad_list=[]
        u0_list=[]
        u_array=[]
        soc_array=[]
        
        #tworzenie wymuszenia
        exprI=0*Heaviside(t-0)
        for i in range(len(step_time)):
            exprI=exprI + step_current[i]*Heaviside(t-step_time[i])
        funcI = lambdify(t, exprI)
        #exprI=step_current[0]*Heaviside(t-step_time[0])+step_current[1]*Heaviside(t-step_time[1])+step_current[2]*Heaviside(t-step_time[2])+step_current[3]*Heaviside(t-step_time[3])
        #funcI = lambdify(t, exprI)
        

        #tworzenie dataframe z wynikami rownania rozniczkowego+tworzenie z tego listy
        tabelka=ode_th.subs(I_li,exprI).subs(data_dict1).numerized(backend='numpy').compute_solution(t_array,[0])
        
        #display(ode_th)
        #display(tabelka)
        
        U_th_list = list(tabelka.iloc[:,1])




        for i in range(n_points):
            prad_list.append(funcI(t_array[i]))
            soc_array.append(float((SOC_eq_bezcalki.rhs.subs(data_dict1)*(-1)*(np.trapz(prad_list[0:i], x=t_array[0:i]))/3600+SOC_init.subs(data_dict1))))

            #tutaj jest dopasowanie ocv w zaleznosci od soc, zamiast ifow
            soc_calk = int(soc_array[i]*100)
            #soc_calk = soc_calk*100
            data_dict1[self.U_oc]= celldata.at[soc_calk, 'OCV']
            data_dict1[self.R_0]=celldata.at[soc_calk, 'R0']

            u0_list.append( prad_list[-1] * data_dict1[self.R_0])
            u_array.append(data_dict1[self.U_oc]-U_th_list[i]-u0_list[i])


        wartosci = [u_array,prad_list,soc_array,t_array]
        
        return wartosci
    

    def _voltage_response(self,data_dict1,celldata,step_time,step_current):

        t=self.t
        U_li= self.U_li
        wyn = self.uth_simulation(data_dict1,celldata,step_time,step_current)
        t_array = wyn[3]
        u_array = wyn[0]

        df_u = pd.DataFrame({t : t_array})
        df_u[U_li] = u_array
        df_u = df_u.set_index(t)

        return df_u
    
    def _soc_level_response(self,data_dict1,celldata,step_time,step_current):
        
        t=self.t
        SOC= self.SOC
        wyn = self.uth_simulation(data_dict1,celldata,step_time,step_current)
        t_array = wyn[3]
        soc_array = wyn[2]

        df_SOC = pd.DataFrame({t : t_array})
        df_SOC[SOC] = soc_array
        df_SOC = df_SOC.set_index(t)

        return df_SOC
    
    
    def _current_forcing(self,data_dict1,celldata,step_time,step_current):
        
        t=self.t
        I_li= self.I_li
        wyn = self.uth_simulation(data_dict1,celldata,step_time,step_current)
        t_array = wyn[3]
        prad_list = wyn[1]

        df_I = pd.DataFrame({t : t_array})
        df_I[I_li] = prad_list
        df_I = df_I.set_index(t)

        return df_I
        
        

#         tdf_u=TimeDataFrame(df_u).set_index(t)
#         tdf_I=TimeDataFrame(df_I).set_index(t)
#         tdf_SOC=TimeDataFrame(df_SOC).set_index(t)


class BatteryCharging(ComposedSystem):
    
    V_OCV = Symbol('V_OCV',positive=True)
    Vd = Symbol('Vd',positive=True)
    Vo = Symbol('Vo',positive=True)
    Ro = Symbol('Ro',positive=True)
    current = Symbol('I_ch', positive=True)
    t0 = Symbol('t_0',positive=True)
    resistance_0 = Symbol('R_0', positive=True)
    resistance_coefficient_alpha = Symbol('R_Alpha', positive=True) #symbol alfy
    resistance_coefficient_beta = Symbol('R_Beta', positive=True) #symbol bety
    resistance_coefficient_gamma = Symbol('R_Gamma', positive=True) #symbol gammy
    resistance_coefficient_blocking = Symbol('R_Omega', positive=True) #symbol omegi
    charger_blocking_resistance = Symbol('R_b', positive=True) 
    resistance  = Symbol('R' , positive= True)
    coil_coefficient = Symbol('L',positive=True)
    capacitor_coefficient= Symbol('C',positive=True)
    electric_charge = dynamicsymbols('q')
    k_p = Symbol ('k_p',positive=True)
    k_d = Symbol ('k_d',positive=True)
    PD = Symbol ('PD',positive=True)
    q_char = Symbol ('q_char',positive=True)

    def __init__(self,
                 V_OCV = V_OCV,
                 Vo = Vo,
                 Vd = Vd,
                 Ro = Ro,
                 current = current,
                 t0 = t0,
                 resistance_0 = resistance_0,
                 resistance_coefficient_alpha = resistance_coefficient_alpha,
                 resistance_coefficient_beta = resistance_coefficient_beta,
                 resistance_coefficient_gamma = resistance_coefficient_gamma,
                 resistance_coefficient_blocking = resistance_coefficient_blocking,
                 charger_blocking_resistance = charger_blocking_resistance,
                 resistance = resistance,
                 coil_coefficient = coil_coefficient,
                 capacitor_coefficient=capacitor_coefficient,
                 electric_charge = electric_charge,
                 k_p = k_p,
                 k_d = k_d,
                 PD = PD,
                 q_char = q_char,
                 qs=[electric_charge],
                 ivar=Symbol('t'),
                 **kwargs):
    
        t=ivar
        self.resistance_0 = resistance_0
        self.Vo = Vo
        self.Vd = Vd
        self.Ro = Ro
        self.resistance_coefficient_alpha =  resistance_coefficient_alpha
        self.resistance_coefficient_beta =  resistance_coefficient_beta
        self.resistance_coefficient_gamma =  resistance_coefficient_gamma
        self.resistance_coefficient_blocking =  resistance_coefficient_blocking
        self.coil_coefficient = coil_coefficient
        self.capacitor_coefficient = capacitor_coefficient
        self.electric_charge = electric_charge
        self.current = current
        self.qs = [self.electric_charge]
        
        ### krzywa U(SOC) w procesie ładowania
        x_char = [0, 50, 100, 150, 200, 250, 300, 350, 400, 450, 500, 550, 600, 650, 700, 750, 800, 900, 1000, 1100, 1200, 1300, 1400, 1500, 1600, 1700, 1800, 1900, 2000, 2100, 2200, 2300, 2400,     2500, 2600, 2700, 2800, 2900, 3000, 3100, 3200, 3300, 3400, 3500, 3600, 3700, 3800, 3900, 4000, 4100, 4200, 4300, 4400, 4500, 4600, 4700, 4800, 4900, 5000, 5100, 5200, 5300, 5400, 5500, 5600, 5700, 5800, 5900, 6000, 6100, 6200, 6300, 6400, 6500, 6600, 6650, 6800, 6850, 6900, 6950, 7000, 7050, 7100, 7150, 7200, 7250, 7300, 7350, 7400, 7450, 7500, 7550, 7600, 7650, 7700, 7750, 7800, 7850, 7900, 7950, 8000, 8050, 8100, 8150, 8200, 8250, 8300, 8350, 8400, 8450, 8500, 8550, 8600, 8650, 8800, 8900, 9400, 9800, 10200, 10600]

        y_char = [0, 0.2, 0.4, 0.6, 0.8, 1, 1.2, 1.4, 1.6, 1.8, 2, 2.2, 2.4, 2.5, 2.55, 2.62, 2.74, 2.86, 2.98, 3.1, 3.22, 3.34, 3.46, 3.58, 3.7, 3.75, 3.801, 3.802, 3.803, 3.804, 3.805, 3.806, 3.807, 3.808, 3.809, 3.81, 3.811, 3.812, 3.813, 3.814, 3.815, 3.816, 3.817, 3.818, 3.819, 3.82, 3.821, 3.822, 3.823, 3.824, 3.825, 3.826, 3.827, 3.828, 3.829, 3.83, 3.831, 3.832, 3.833, 3.834, 3.835, 3.836, 3.837, 3.838, 3.839, 3.84, 3.841, 3.842, 3.843, 3.844, 3.845, 3.846, 3.847, 3.848, 3.849, 3.85, 3.86, 3.87, 3.88, 3.89, 3.9, 3.91, 3.92, 3.93, 3.94, 3.95, 3.96, 3.97, 3.98, 3.99, 4, 4.01, 4.02, 4.05, 4.1, 4.15, 4.2, 4.22, 4.24, 4.26, 4.28, 4.3, 4.32, 4.34, 4.36, 4.38, 4.4, 4.42, 4.44, 4.46, 4.48, 4.5, 4.52, 4.6, 4.8, 5, 5.5, 6, 6.5, 7]
        char = np.polyfit(x_char,y_char,11)
        
        self.V_OCV  = char[0]*self.electric_charge**11 + char[1]*self.electric_charge**10 + char[2]*self.electric_charge**9 + char[3]*self.electric_charge**8 + char[4]*self.electric_charge**7 + char[5]*self.electric_charge**6 + char[6]*self.electric_charge**5 + char[7]*self.electric_charge**4 + char[8]*self.electric_charge**3 + char[9]*self.electric_charge**2 + char[10]*self.electric_charge + char[11]  
        
        
        self.charger_blocking_resistance = exp((self.electric_charge + (-1 * self.q_char - 250))/50) #self.resistance_coefficient_blocking
        
        self.resistance = self.resistance_0 + exp((self.resistance_coefficient_alpha - self.electric_charge)/self.resistance_coefficient_beta) +  exp((self.electric_charge + self.resistance_coefficient_gamma)/5550 ) 
        
        self.material_point_L = MaterialPoint(self.coil_coefficient,self.electric_charge, qs=[self.electric_charge])
        self.damper_R= Damper(self.resistance,self.electric_charge, qs=[self.electric_charge])
        self.damper_R_blocking= Damper(self.charger_blocking_resistance,self.electric_charge, qs=[self.electric_charge])
        self.force_OCV = Force(self.V_OCV,self.electric_charge,qs=[self.electric_charge])     
        self.spring_C = Spring(1/self.capacitor_coefficient, self.electric_charge, qs=[self.electric_charge])
        
        
        system = self.damper_R + self.damper_R_blocking + self.force_OCV + self.material_point_L + self.spring_C
        #super().__init__(system(qs),**kwargs)

        ###### regulator PD
        
        self.q_char = q_char
        self.t0 = ((self.q_char*0.0002777)/self.current)*3600 # ładunek żądany/stała(zamiana C na AH) dzielone przez prad ladowania  i zamiana na sekundy
        self.charger_slope = 0.001
        self.k_p = k_p 
        self.k_d = k_d   
        
        pd = Force((self.k_p* (S.One/2-atan(self.charger_slope*(self.ivar-self.t0))/pi)) *(self.current - self.electric_charge.diff(t)), self.electric_charge, qs = [self.electric_charge]  ) + Force((self.k_d * (S.One/2-atan(self.charger_slope*(self.ivar-self.t0))/pi)) * (self.current - self.electric_charge.diff(t)).diff(t),self.electric_charge ,qs = [self.electric_charge])

        system_pid = system+pd(qs)
        super().__init__(system_pid,**kwargs)

    def symbols_description(self):
        self.sym_desc_dict = {
            self.resistance_0: r'Rezystancja podstawowa',
            self.resistance: r'Rezystor (SOC)',
            self.resistance_coefficient_alpha: r'Współczynnik rezystancji /alpha',
            self.resistance_coefficient_beta: r'Współczynnik rezystancji /beta',
            self.resistance_coefficient_gamma: r'Współczynnik rezystancji /gamma',
            self.charger_blocking_resistance: r'Rezystancja blokująca',
            self.resistance_coefficient_blocking: r'Współczynnik rezystancji blokującej', 
            self.capacitor_coefficient: r'Kondensator',
            self.coil_coefficient: r'Cewka indukcyjna',
            self.V_OCV: r'Napięcie',
            self.Vo:r'Napięcie',
            self.electric_charge: r'Ładunek',
            self.electric_charge.diff(t): r'Prąd',
            self.current: r'Prąd ładowania',
            self.t0: r'Czas ładowania',
            self.pd: r'Regulator PD',
            self.k_p: r'Człon proporcjonalny regulatora',
            self.k_d: r'Człon różniczkujący regulatora',
            self.ivar: r'Czas',

        }
        return self.sym_desc_dict


class BatteryModeling(ComposedSystem):
    
    V_OCV = Symbol('V_OCV',positive=True)
    current = Symbol('I_ch', positive=True)
    t0 = Symbol('t_0',positive=True)
    resistance_0 = Symbol('R_0', positive=True)
    resistance_coefficient_alpha = Symbol('R_Alpha', positive=True) #symbol alfy
    resistance_coefficient_beta = Symbol('R_Beta', positive=True) #symbol bety
    resistance_coefficient_gamma = Symbol('R_Gamma', positive=True) #symbol gammy
    resistance_coefficient_blocking = Symbol('R_Omega', positive=True) #symbol omegi
    charger_blocking_resistance = Symbol('R_b', positive=True) 
    resistance  = Symbol('R' , positive= True)
    coil_coefficient = Symbol('L',positive=True)
    capacitor_coefficient= Symbol('C',positive=True)
    electric_charge = dynamicsymbols('q')
    pid = Symbol ('PD',positive=True)
    q_char = Symbol ('q_char',positive=True)
    k_p = Symbol ('k_p',positive=True)
    k_d = Symbol ('k_d',positive=True)
    PD = Symbol ('PD',positive=True)
    n_0 = Symbol('n_0',positive=True)
    n_1 = Symbol('n_0',positive=True)
    n_2 = Symbol('n_0',positive=True)
    n_3 = Symbol('n_0',positive=True)
    n_4 = Symbol('n_0',positive=True)
    n_5 = Symbol('n_0',positive=True)
    n_6 = Symbol('n_0',positive=True)
    n_7 = Symbol('n_0',positive=True)
    n_8 = Symbol('n_0',positive=True)
    n_9= Symbol('n_0',positive=True)
    n_10 = Symbol('n_0',positive=True)
    n_11 = Symbol('n_0',positive=True)
    charger_slope = Symbol('chs',positive=True)

    def __init__(self,
                 V_OCV = V_OCV,
                 current = current,
                 t0 = t0,
                 resistance_0 = resistance_0,
                 resistance_coefficient_alpha = resistance_coefficient_alpha,
                 resistance_coefficient_beta = resistance_coefficient_beta,
                 resistance_coefficient_gamma = resistance_coefficient_gamma,
                 resistance_coefficient_blocking = resistance_coefficient_blocking,
                 charger_blocking_resistance = charger_blocking_resistance,
                 resistance = resistance,
                 coil_coefficient = coil_coefficient,
                 capacitor_coefficient=capacitor_coefficient,
                 electric_charge = electric_charge,
                 pid = pid,
                 q_char = q_char,
                 k_p = k_p,
                 k_d = k_d,
                 PD = PD,
                 n_0 = n_0,
                 n_1 = n_1,
                 n_2 = n_2,
                 n_3 = n_3,
                 n_4 = n_4,
                 n_5 = n_5,
                 n_6 = n_6,
                 n_7 = n_7,
                 n_8 = n_8,
                 n_9 = n_9,
                 n_10 = n_10,
                 n_11 = n_11,
                 charger_slope = charger_slope,
                 qs=[electric_charge],
                 ivar=Symbol('t'),
                 **kwargs):
    
        t=ivar
        self.resistance_0 = resistance_0
        self.resistance_coefficient_alpha =  resistance_coefficient_alpha
        self.resistance_coefficient_beta =  resistance_coefficient_beta
        self.resistance_coefficient_gamma =  resistance_coefficient_gamma
        self.resistance_coefficient_blocking =  resistance_coefficient_blocking
        self.coil_coefficient = coil_coefficient
        self.capacitor_coefficient = capacitor_coefficient
        self.electric_charge = electric_charge
        self.current = current
        self.qs = [self.electric_charge]
        
        ### krzywa U(SOC) w procesie ładowania
#         x_char = [0, 50, 100, 150, 200, 250, 300, 350, 400, 450, 500, 550, 600, 650, 700, 750, 800, 900, 1000, 1100, 1200, 1300, 1400, 1500, 1600, 1700, 1800, 1900, 2000, 2100, 2200, 2300, 2400,     2500, 2600, 2700, 2800, 2900, 3000, 3100, 3200, 3300, 3400, 3500, 3600, 3700, 3800, 3900, 4000, 4100, 4200, 4300, 4400, 4500, 4600, 4700, 4800, 4900, 5000, 5100, 5200, 5300, 5400, 5500, 5600, 5700, 5800, 5900, 6000, 6100, 6200, 6300, 6400, 6500, 6600, 6650, 6800, 6850, 6900, 6950, 7000, 7050, 7100, 7150, 7200, 7250, 7300, 7350, 7400, 7450, 7500, 7550, 7600, 7650, 7700, 7750, 7800, 7850, 7900, 7950, 8000, 8050, 8100, 8150, 8200, 8250, 8300, 8350, 8400, 8450, 8500, 8550, 8600, 8650, 8800, 8900, 9400, 9800, 10200, 10600]

#         y_char = [0, 0.2, 0.4, 0.6, 0.8, 1, 1.2, 1.4, 1.6, 1.8, 2, 2.2, 2.4, 2.5, 2.55, 2.62, 2.74, 2.86, 2.98, 3.1, 3.22, 3.34, 3.46, 3.58, 3.7, 3.75, 3.801, 3.802, 3.803, 3.804, 3.805, 3.806, 3.807, 3.808, 3.809, 3.81, 3.811, 3.812, 3.813, 3.814, 3.815, 3.816, 3.817, 3.818, 3.819, 3.82, 3.821, 3.822, 3.823, 3.824, 3.825, 3.826, 3.827, 3.828, 3.829, 3.83, 3.831, 3.832, 3.833, 3.834, 3.835, 3.836, 3.837, 3.838, 3.839, 3.84, 3.841, 3.842, 3.843, 3.844, 3.845, 3.846, 3.847, 3.848, 3.849, 3.85, 3.86, 3.87, 3.88, 3.89, 3.9, 3.91, 3.92, 3.93, 3.94, 3.95, 3.96, 3.97, 3.98, 3.99, 4, 4.01, 4.02, 4.05, 4.1, 4.15, 4.2, 4.22, 4.24, 4.26, 4.28, 4.3, 4.32, 4.34, 4.36, 4.38, 4.4, 4.42, 4.44, 4.46, 4.48, 4.5, 4.52, 4.6, 4.8, 5, 5.5, 6, 6.5, 7]
#         char = np.polyfit(x_char,y_char,11)
        
        self.V_OCV  = n_0*self.electric_charge**11 + n_1*self.electric_charge**10 + n_2*self.electric_charge**9 + n_3*self.electric_charge**8 + n_4*self.electric_charge**7 + n_5*self.electric_charge**6 + n_6*self.electric_charge**5 + n_7*self.electric_charge**4 + n_8*self.electric_charge**3 + n_9*self.electric_charge**2 + n_10*self.electric_charge + n_11  
        
        
        self.charger_blocking_resistance = exp((self.electric_charge + (-1 * self.q_char - 250))/50) #self.resistance_coefficient_blocking
        self.resistance = self.resistance_0 + exp((self.resistance_coefficient_alpha - self.electric_charge)/self.resistance_coefficient_beta) +  exp((self.electric_charge + self.resistance_coefficient_gamma)/self.resistance_coefficient_beta) 
        self.material_point_L = MaterialPoint(self.coil_coefficient,self.electric_charge, qs=[self.electric_charge])
        self.damper_R= Damper(self.resistance,self.electric_charge, qs=[self.electric_charge])
        self.force_OCV = Force(self.V_OCV,self.electric_charge,qs=[self.electric_charge])     
        self.spring_C = Spring(1/self.capacitor_coefficient, self.electric_charge, qs=[self.electric_charge])
        
        
        system =self.damper_R  +  self.force_OCV + self.material_point_L + self.spring_C 
        #super().__init__(system(qs),**kwargs)

        ###### regulator PD
        self.damper_Rb= Damper(self.charger_blocking_resistance,self.electric_charge, qs=[self.electric_charge])
        self.q_char = q_char
        self.t0 = ((self.q_char)/self.current) # ładunek żądany/stała(zamiana C na AH) dzielone przez prad ladowania  i zamiana na sekundy
        self.charger_slope = charger_slope
        self.k_p = k_p 
        self.k_d = k_d
        
        self.pid = ((self.k_p* (S.One/2-atan(self.charger_slope*(self.ivar-self.t0))/pi)) *(self.current - self.electric_charge.diff(t))) + ((self.k_d * (S.One/2-atan(self.charger_slope*(self.ivar-self.t0))/pi)) * (self.current - self.electric_charge.diff(t)).diff(t))
        
        pd = Force((self.k_p* (S.One/2-atan(self.charger_slope*(self.ivar-self.t0))/pi)) *(self.current - self.electric_charge.diff(t)), self.electric_charge, qs = [self.electric_charge]  ) + Force((self.k_d * (S.One/2-atan(self.charger_slope*(self.ivar-self.t0))/pi)) * (self.current - self.electric_charge.diff(t)).diff(t),self.electric_charge ,qs = [self.electric_charge]) + self.damper_Rb
        
        system_pid = system+pd(qs)
        super().__init__(system_pid,**kwargs)
        
    def symbols_description(self):
        self.sym_desc_dict = {
            self.resistance_0: r'Rezystancja podstawowa',
            self.resistance: r'Rezystor (SOC)',
            self.resistance_coefficient_alpha: r'Współczynnik rezystancji \\alpha',
            self.resistance_coefficient_beta: r'Współczynnik rezystancji \\beta',
            self.resistance_coefficient_gamma: r'Współczynnik rezystancji \\gamma',
            self.resistance_coefficient_blocking: r'Współczynnik rezystancji blokującej', 
            self.capacitor_coefficient: r'Kondensator',
            self.coil_coefficient: r'Cewka indukcyjna',
            self.V_OCV: r'Napięcie',
            self.electric_charge: r'Ładunek',
            self.electric_charge.diff(t): r'Prąd',
            self.current: r'Prąd ładowania',
            self.ivar: r'czas',
            
        }
        return self.sym_desc_dict
    
    
    
class CircutRl(ComposedSystem):
    """
    A class that determines the equation of an electrical circuit in an RL system
    """
    # scheme_name = 'engine.png'
    # real_name = 'engine_real.PNG'
    

    resistance=Symbol('R', positive=True)
    inductance=Symbol('L', positive=True)
    q0=dynamicsymbols('q_c')
    qs=dynamicsymbols('qs')
    frame=Symbol('frame', positive=True)
    ivar=Symbol('t')
    
    
    
    def __init__(self,
                 resistance=None,
                 inductance=None,
                 ivar=None,
                 q0=None,
                 qs=None,
                 frame=None,
                 z=None,
                 **kwargs):

        
        
        if resistance is not None: self.resistance = resistance
        if inductance is not None: self.inductance = inductance
        if ivar is not None: self.ivar = ivar
        if z is not None: self.z = z
        if q0 is not None: self.q0 = q0
        if qs is not None: self.qs = qs
        if frame is not None: self.frame = frame

        self.qs = [self.q0]

        self._init_from_components(**kwargs)

    @property
    def components(self):

        components = {}
        
        self.resistor = Resistor(self.resistance, self.q0,  qs=self.qs,ivar=self.ivar , frame=base_frame)('resistor')
        self.inductor = Inductor(self.inductance, self.q0, ivar=self.ivar, qs=self.qs, frame=base_frame)('inductor')
        
        components['resistor'] = self.resistor
        components['inductor'] = self.inductor

        
        return components
        
    def symbols_description(self):
        self.sym_desc_dict = {
            self.resistance: r'resistance of resistor',
            self.inductance: r'inductance of inductor',
        }

        return self.sym_desc_dict
    
    
    
class CircutRC(ComposedSystem):
    """
    A class that determines the equation of an electrical circuit in an RC system
    """
    # scheme_name = 'engine.png'
    # real_name = 'engine_real.PNG'
    

    resistance=Symbol('R', positive=True)
    capacity=Symbol('C', positive=True)
    q0=dynamicsymbols('q_c')
    qs=dynamicsymbols('qs')
    frame=Symbol('frame', positive=True)
    ivar=Symbol('t')
    
    
    
    def __init__(self,
                 resistance=None,
                 capacity=None,
                 ivar=None,
                 q0=None,
                 qs=None,
                 frame=None,
                 z=None,
                 **kwargs):

        
        
        if resistance is not None: self.resistance = resistance
        if capacity is not None: self.capacity = capacity
        if ivar is not None: self.ivar = ivar
        if z is not None: self.z = z
        if q0 is not None: self.q0 = q0
        if qs is not None: self.qs = qs
        if frame is not None: self.frame = frame

        self.qs = [self.q0]

        self._init_from_components(**kwargs)

    @property
    def components(self):

        components = {}
        
        self.resistor = Resistor(self.resistance, self.q0,  qs=self.qs,ivar=self.ivar , frame=base_frame)('resistor')
        self.capacitor = Capacitor(self.capacity, self.q0, ivar=self.ivar, qs=self.qs, frame=base_frame)('capacitor')
        
        components['resistor'] = self.resistor
        components['capacitor'] = self.capacitor

        
        return components
        
    def symbols_description(self):
        self.sym_desc_dict = {
            self.resistance: r'resistance of resistor',
            self.capacity: r'capacity of capacitor',
        }

        return self.sym_desc_dict
    
    
    
class CircuitRLC(ComposedSystem):
    """
    A class that determines the equation of an electrical circuit in an RLC system
    """
    # scheme_name = 'engine.png'
    # real_name = 'engine_real.PNG'
    

    resistance=Symbol('R', positive=True)
    inductance=Symbol('L', positive=True)
    capacity=Symbol('C', positive=True)
    q0=dynamicsymbols('q')
    frame=Symbol('frame', positive=True)
    ivar=Symbol('t')
    
    
    
    def __init__(self,
                 resistance=None,
                 inductance=None,
                 capacity=None,
                 ivar=None,
                 q0=None,
                 frame=None,
                 z=None,
                 **kwargs):

        
        
        if resistance is not None: self.resistance = resistance
        if inductance is not None: self.inductance = inductance
        if capacity is not None: self.capacity = capacity
        if ivar is not None: self.ivar = ivar
        if z is not None: self.z = z
        if q0 is not None: self.q0 = q0
        if frame is not None: self.frame = frame

        self.qs = [self.q0]

        self._init_from_components(**kwargs)

    @property
    def components(self):

        components = {}
        
        self.resistor = Resistor(self.resistance, self.q0,  qs=self.qs,ivar=self.ivar , frame=base_frame)('resistor')
        self.inductor = Inductor(self.inductance, self.q0, ivar=self.ivar, qs=self.qs, frame=base_frame)('inductor')
        self.capacitor = Capacitor(self.capacity, self.q0, ivar=self.ivar, qs=self.qs, frame=base_frame)('capacitor')
        
        components['resistor'] = self.resistor
        components['inductor'] = self.inductor
        components['capacitor'] = self.capacitor

        
        return components
        
    def symbols_description(self):
        self.sym_desc_dict = {
            self.resistance: r'resistance of resistor',
            self.inductance: r'inductance of inductor',
            self.capacity: r'capacity of capacitor',
        }

        return self.sym_desc_dict
    
class CircuitRLCWithPWM(CircuitRLC):
    """
    A class that determines the equation of an electrical circuit in an RLC system
    """
    # scheme_name = 'engine.png'
    # real_name = 'engine_real.PNG'
    

    resistance=Symbol('R', positive=True)
    inductance=Symbol('L', positive=True)
    capacity=Symbol('C', positive=True)
    U=Symbol('U',positive=True)
    eps=Symbol('varepsilon',postive=True)
    rho=Symbol('rho',positive=True)
    delta=Symbol('delta',positive=True)
    T=Symbol('T',positive=True)
    q0=dynamicsymbols('q')
    omega=Symbol('omega',positive=True)
    ivar=Symbol('t')
    
    
    
    def __init__(self,
                 resistance=None,
                 inductance=None,
                 capacity=None,
                 U=None,
                 eps=None,
                 rho=None,
                 delta=None,
                 T=None,
                 ivar=None,
                 q0=None,
                 frame=None,
                 z=None,
                 **kwargs):

        
        
        if resistance is not None: self.resistance = resistance
        if inductance is not None: self.inductance = inductance
        if capacity is not None: self.capacity = capacity
        if U is not None: self.U=U
        if eps is not None: self.eps=eps
        if rho is not None: self.rho=rho
        if delta is not None: self.delta=delta
        if T is not None: self.T=T
        if ivar is not None: self.ivar = ivar
        if z is not None: self.z = z
        if q0 is not None: self.q0 = q0
        if frame is not None: self.frame = frame

        self.qs = [self.q0]

        self._init_from_components(**kwargs)

    @property
    def components(self):

        components = {}
        self.r_var = self.resistance*(1+self.eps*self.rho  ) 
        
        
        self.resistor = Resistor(self.r_var, self.q0,  qs=self.qs,ivar=self.ivar , frame=base_frame)('resistor')
        self.inductor = Inductor(self.inductance, self.q0, ivar=self.ivar, qs=self.qs, frame=base_frame)('inductor')
        self.capacitor = Capacitor(self.capacity/self.delta/self.eps, self.q0, ivar=self.ivar, qs=self.qs, frame=base_frame)('capacitor')
        self.voltage= VoltageSource(self.U,self.q0,ivar=self.ivar,qs=self.qs)
        
        components['resistor'] = self.resistor
        components['inductor'] = self.inductor
        components['capacitor'] = self.capacitor
        components['voltage'] = self.voltage

        
        return components
        
    def symbols_description(self):
        
        self.sym_desc_dict = {
            self.resistance: r'resistance',
            self.inductance: r'inductance',
            self.capacity: r'capacitance',
            self.U:r'source voltage',
            self.eps:r'small parameter',
            self.rho:r'formula representing resistance variation',
            self.q0:r'electric charge',
            self.q0.diff(self.ivar):r'current',
            self.q0.diff(self.ivar,self.ivar):r'current rate of change'

        }

        return self.sym_desc_dict
    
    def para(self, angle=2*pi):
        para_series_heave =  10*Heaviside(sin(2*pi/(self.T)*self.ivar),0.5)
        new_eq=self.subs(self.rho,para_series_heave)
        return new_eq
    
    def wave(self):
        
        wave1=(1*(sin(2*pi*self.ivar/self.T))+2.0)*(1/2 + 1/2*sign(sin(2*pi*self.ivar/self.T)))
        wave2=(1*(-sin(2*pi*self.ivar/self.T))-3.0)*(1/2+ 1/2*sign(-sin(2*pi*self.ivar/self.T)))
        waves=wave1+wave2
#         wave1=(100*(sin(self.ivar/self.T))+1000)*Heaviside(sin(self.ivar/self.T))
#         wave2=(100*(-sin(self.ivar/self.T)))*Heaviside(-sin(self.ivar/self.T))
        waves=wave1+wave2
        new_eq = self.subs(self.rho,waves)

        return new_eq
        
    def rect(self,no=6,numerical=False):
        t=self.ivar
        omg = self.omega
        rectangular=5*((1/2+1/2*sign(sin(2*pi*self.ivar/self.self.T)))-S.Half)
#         rectangular_func = lambdify((t, T), rectangular)
#         rectangular_values = rectangular_func(t_values, T_value)
        
#         trig=sum([Heaviside(omg*t-1) + 0* Heaviside(omg*t-2)  for ind in range(no)])
#         new_eq=self.subs(self.rho,trig)
        new_eq=self.subs(self.rho,rectangular)
        
        return new_eq
    
    def approx_rect(self,no=6,numerical=False):
        
        if numerical is True:
            amps_list = [2.27348466531425,0, 0.757408805199249,0, 0.453942816897038,0, 0.323708002807428,0, 0.25121830779797,0, 0.204977919963796,0, 0.172873394602606,0, 0.149252079729775,0, 0.131121653619234,0, 0.116749954968057,0]
        else:
            amps_list = symbols(f'a_0:{no}')
        rectangular_approx = sum([amp*2/sqrt(2)*sin((2*(no)+1)*2*pi*self.ivar/self.T) for no,amp in enumerate(amps_list[0:])])
        new_eq=self.subs({self.rho:rectangular_approx})
        
        return new_eq
    
#     def ode_with_delta(self):
#         delta=Symbol('delta', positive=True)
#         eps =Symbol('varepsilon', positive=True)
        
#         with_delta = self._eoms[0]+self.resistance*eps*delta*self.q0.diff(self.ivar)
#         delta_sys = type(self._ode_system)(with_delta, Matrix([self.q0]), ivar=self.ivar, ode_order=2)

#         return delta_sys

class CircuitRLCWithHeavisidePWM(CircuitRLCWithPWM):
    """
    A class that determines the equation of an electrical circuit in an RLC system
    """
    # scheme_name = 'engine.png'
    # real_name = 'engine_real.PNG'
    

    resistance=Symbol('R', positive=True)
    inductance=Symbol('L', positive=True)
    capacity=Symbol('C', positive=True)
    R_min=Symbol('R_min',positive=True)
    U=Symbol('U',positive=True)
    eps=Symbol('varepsilon',postive=True)
    rho=Symbol('rho',positive=True)
    delta=Symbol('delta',positive=True)
    T=Symbol('T',positive=True)
    q0=dynamicsymbols('q')
    omega=Symbol('omega',positive=True)
    ivar=Symbol('t')
    
    
    
    def __init__(self,
                 resistance=None,
                 R_min=None,
                 inductance=None,
                 capacity=None,
                 U=None,
                 eps=None,
                 rho=None,
                 delta=None,
                 T=None,
                 ivar=None,
                 q0=None,
                 frame=None,
                 z=None,
                 **kwargs):

        
        
        if resistance is not None: self.resistance = resistance
        if R_min is not None: self.R_min=R_min
        if inductance is not None: self.inductance = inductance
        if capacity is not None: self.capacity = capacity
        if U is not None: self.U=U
        if eps is not None: self.eps=eps
        if rho is not None: self.rho=rho
        if delta is not None: self.delta=delta
        if T is not None: self.T=T
        if ivar is not None: self.ivar = ivar
        if z is not None: self.z = z
        if q0 is not None: self.q0 = q0
        if frame is not None: self.frame = frame

        self.qs = [self.q0]

        self._init_from_components(**kwargs)

    @property
    def components(self):

        components = {}
        self.r_var = self.resistance*(1+self.eps*self.rho  ) 
        
        
        self.resistor = Resistor(self.R_min+self.resistance*Heaviside(sin(2*pi*self.ivar/self.T)), self.q0,  qs=self.qs,ivar=self.ivar , frame=base_frame)('resistor')
        self.inductor = Inductor(self.inductance, self.q0, ivar=self.ivar, qs=self.qs, frame=base_frame)('inductor')
        self.capacitor = Capacitor(self.capacity, self.q0, ivar=self.ivar, qs=self.qs, frame=base_frame)('capacitor')
        self.voltage= VoltageSource(self.U,self.q0,ivar=self.ivar,qs=self.qs)
        
        components['resistor'] = self.resistor
        components['inductor'] = self.inductor
        components['capacitor'] = self.capacitor
        components['voltage'] = self.voltage

        
        return components
    
    def symbols_description(self):
        
        self.sym_desc_dict = {
            self.resistance: r'amplitude of the variable part of the resistance',
            self.R_min:r'constant value of the resistance',
            self.inductance: r'inductance',
            self.capacity: r'capacitance',
            self.U:r'source voltage',
            self.q0:r'electric charge',
            self.q0.diff(self.ivar):r'current',
            self.q0.diff(self.ivar,self.ivar):r'current rate of change'

        }

        return self.sym_desc_dict
        

class BatteryCharging(ComposedSystem):
    
    V_OCV = Symbol('V_OCV',positive=True)
    current = Symbol('I_ch', positive=True)
    t0 = Symbol('t_0',positive=True)
    Tk = Symbol('T_k',positive=True)
    A = Symbol('A',positive=True)
    Phi = Symbol('\Phi', positive = True)
    resistance_0 = Symbol('R_0', positive=True)
    resistance_coefficient_alpha = Symbol('R_Alpha', positive=True) #symbol alfy
    resistance_coefficient_beta = Symbol('R_Beta', positive=True) #symbol bety
    resistance_coefficient_gamma = Symbol('R_Gamma', positive=True) #symbol gammy
    resistance_coefficient_blocking = Symbol('R_Omega', positive=True) #symbol omegi
    charger_blocking_resistance = Symbol('R_b', positive=True) 
    resistance  = Symbol('R' , positive= True)
    coil_coefficient = Symbol('L',positive=True)
    capacitor_coefficient= Symbol('C',positive=True)
    electric_charge = dynamicsymbols('q')
    k_p = Symbol ('k_p',positive=True)
    k_d = Symbol ('k_d',positive=True)
    PD = Symbol ('PD',positive=True)
    q_char = Symbol ('q_char',positive=True)

    def __init__(self,
                 V_OCV = V_OCV,
                 current = current,
                 t0 = t0,
                 Tk = Tk,
                 A = A,
                 Phi = Phi,
                 resistance_0 = resistance_0,
                 resistance_coefficient_alpha = resistance_coefficient_alpha,
                 resistance_coefficient_beta = resistance_coefficient_beta,
                 resistance_coefficient_gamma = resistance_coefficient_gamma,
                 resistance_coefficient_blocking = resistance_coefficient_blocking,
                 charger_blocking_resistance = charger_blocking_resistance,
                 resistance = resistance,
                 coil_coefficient = coil_coefficient,
                 capacitor_coefficient=capacitor_coefficient,
                 electric_charge = electric_charge,
                 k_p = k_p,
                 k_d = k_d,
                 PD = PD,
                 q_char = q_char,
                 qs=[electric_charge],
                 ivar=Symbol('t'),
                 **kwargs):
    
        t=ivar
        self.resistance_0 = resistance_0
        self.Tk= Tk
        self.A = A
        self.Phi = Phi
        self.resistance_coefficient_alpha =  resistance_coefficient_alpha
        self.resistance_coefficient_beta =  resistance_coefficient_beta
        self.resistance_coefficient_gamma =  resistance_coefficient_gamma
        self.resistance_coefficient_blocking =  resistance_coefficient_blocking
        self.coil_coefficient = coil_coefficient
        self.capacitor_coefficient = capacitor_coefficient
        self.electric_charge = electric_charge
        self.current = current
        self.qs = [self.electric_charge]
        
        ### krzywa U(SOC) w procesie ładowania
        x_char = [0, 50, 100, 150, 200, 250, 300, 350, 400, 450, 500, 550, 600, 650, 700, 750, 800, 900, 1000, 1100, 1200, 1300, 1400, 1500, 1600, 1700, 1800, 1900, 2000, 2100, 2200, 2300, 2400,     2500, 2600, 2700, 2800, 2900, 3000, 3100, 3200, 3300, 3400, 3500, 3600, 3700, 3800, 3900, 4000, 4100, 4200, 4300, 4400, 4500, 4600, 4700, 4800, 4900, 5000, 5100, 5200, 5300, 5400, 5500, 5600, 5700, 5800, 5900, 6000, 6100, 6200, 6300, 6400, 6500, 6600, 6650, 6800, 6850, 6900, 6950, 7000, 7050, 7100, 7150, 7200, 7250, 7300, 7350, 7400, 7450, 7500, 7550, 7600, 7650, 7700, 7750, 7800, 7850, 7900, 7950, 8000, 8050, 8100, 8150, 8200, 8250, 8300, 8350, 8400, 8450, 8500, 8550, 8600, 8650, 8800, 8900, 9400, 9800, 10200, 10600]

        y_char = [0, 0.2, 0.4, 0.6, 0.8, 1, 1.2, 1.4, 1.6, 1.8, 2, 2.2, 2.4, 2.5, 2.55, 2.62, 2.74, 2.86, 2.98, 3.1, 3.22, 3.34, 3.46, 3.58, 3.7, 3.75, 3.801, 3.802, 3.803, 3.804, 3.805, 3.806, 3.807, 3.808, 3.809, 3.81, 3.811, 3.812, 3.813, 3.814, 3.815, 3.816, 3.817, 3.818, 3.819, 3.82, 3.821, 3.822, 3.823, 3.824, 3.825, 3.826, 3.827, 3.828, 3.829, 3.83, 3.831, 3.832, 3.833, 3.834, 3.835, 3.836, 3.837, 3.838, 3.839, 3.84, 3.841, 3.842, 3.843, 3.844, 3.845, 3.846, 3.847, 3.848, 3.849, 3.85, 3.86, 3.87, 3.88, 3.89, 3.9, 3.91, 3.92, 3.93, 3.94, 3.95, 3.96, 3.97, 3.98, 3.99, 4, 4.01, 4.02, 4.05, 4.1, 4.15, 4.2, 4.22, 4.24, 4.26, 4.28, 4.3, 4.32, 4.34, 4.36, 4.38, 4.4, 4.42, 4.44, 4.46, 4.48, 4.5, 4.52, 4.6, 4.8, 5, 5.5, 6, 6.5, 7]
        char = np.polyfit(x_char,y_char,11)
        
        self.V_OCV  = char[0]*self.electric_charge**11 + char[1]*self.electric_charge**10 + char[2]*self.electric_charge**9 + char[3]*self.electric_charge**8 + char[4]*self.electric_charge**7 + char[5]*self.electric_charge**6 + char[6]*self.electric_charge**5 + char[7]*self.electric_charge**4 + char[8]*self.electric_charge**3 + char[9]*self.electric_charge**2 + char[10]*self.electric_charge + char[11]  
        
        
        self.charger_blocking_resistance = exp((self.electric_charge + (-1 * self.q_char - 250))/50) #self.resistance_coefficient_blocking
        
        self.resistance = self.resistance_0 + exp((self.resistance_coefficient_alpha - self.electric_charge)/self.resistance_coefficient_beta) +  exp((self.electric_charge + self.resistance_coefficient_gamma)/5550 ) 
        
        self.material_point_L = MaterialPoint(self.coil_coefficient,self.electric_charge, qs=[self.electric_charge])
        self.damper_R= Damper(self.resistance,self.electric_charge, qs=[self.electric_charge])
        self.damper_R_blocking= Damper(self.charger_blocking_resistance,self.electric_charge, qs=[self.electric_charge])
        self.force_OCV = Force(self.V_OCV,self.electric_charge,qs=[self.electric_charge])     
        self.spring_C = Spring(1/self.capacitor_coefficient, self.electric_charge, qs=[self.electric_charge])
        
        
        system = self.damper_R + self.damper_R_blocking + self.force_OCV + self.material_point_L + self.spring_C
        #super().__init__(system(qs),**kwargs)

        ###### regulator PD
        
        self.q_char = q_char
        self.t0 = ((self.q_char*0.0002777)/self.current)*3600 # ładunek żądany/stała(zamiana C na AH) dzielone przez prad ladowania  i zamiana na sekundy
        self.charger_slope = 0.001
        self.k_p = k_p 
        self.k_d = k_d   
        
        pd = Force((self.k_p* (S.One/2-atan(self.charger_slope*(self.ivar-self.t0))/pi)) *(self.current - self.electric_charge.diff(t)), self.electric_charge, qs = [self.electric_charge]  ) + Force((self.k_d * (S.One/2-atan(self.charger_slope*(self.ivar-self.t0))/pi)) * (self.current - self.electric_charge.diff(t)).diff(t),self.electric_charge ,qs = [self.electric_charge])
        
        system_pid = system+pd(qs)
        super().__init__(system_pid,**kwargs)
        
    def symbols_description(self):
        self.sym_desc_dict = {
            self.resistance_0: r'Rezystancja podstawowa',
            self.resistance: r'Rezystor (SOC)',
            self.resistance_coefficient_alpha: r'Współczynnik rezystancji /alpha',
            self.resistance_coefficient_beta: r'Współczynnik rezystancji /beta',
            self.resistance_coefficient_gamma: r'Współczynnik rezystancji /gamma',
            self.charger_blocking_resistance: r'Rezystancja blokująca',
            self.resistance_coefficient_blocking: r'Współczynnik rezystancji blokującej', 
            self.capacitor_coefficient: r'Kondensator',
            self.coil_coefficient: r'Cewka indukcyjna',
            self.V_OCV: r'Napięcie',
            self.electric_charge: r'Ładunek',
            self.electric_charge.diff(t): r'Prąd',
            self.current: r'Prąd ładowania',
            self.t0: r'Czas ładowania',
            self.PD: r'Regulator PD',
            self.k_p: r'Człon proporcjonalny regulatora',
            self.k_d: r'Człon różniczkujący regulatora',
            self.ivar: r'Czas',
            
        }
        return self.sym_desc_dict


